from insights.parsers.lvm import LvmConf
from insights.tests import context_wrap

LVM_CONF = """
locking_type = 1
#locking_type = 2
# volume_list = [ "vg1", "vg2/lvol1", "@tag1", "@*" ]
volume_list = [ "vg2", "vg3/lvol3", "@tag2", "@*" ]
# filter = [ "a|loop|", "r|/dev/hdc|", "a|/dev/ide|", "r|.*|" ]

filter = [ "r/sda[0-9]*$/",  "a/sd.*/" ]
filter = [ "a/sda[0-9]*$/",  "r/sd.*/" ] #Required for EMC PP - Do Not Modify this Line
shell {
history_size = 100
}

test_bad_json = [ "partial
""".strip()


def test_lvm_conf():
    lvm_conf_output = LvmConf(context_wrap(LVM_CONF))
    assert lvm_conf_output["locking_type"] == 1
    assert lvm_conf_output["volume_list"] == ['vg2', 'vg3/lvol3', '@tag2', '@*']
    assert lvm_conf_output["filter"] == ['a/sda[0-9]*$/', 'r/sd.*/']
    assert lvm_conf_output['test_bad_json'] == '[ "partial'



"""
Components identify Cloud Provider
===================================

The ``Is*`` component in this module is valid if the
:py:class:`insights.combiners.cloud_provider.CloudProvider` combiner indicates
the host is from the specific Cloud Provider.  Otherwise, it raises a
:py:class:`insights.core.exceptions.SkipComponent` to prevent dependent components from
executing.

"""
from insights.combiners.cloud_provider import CloudProvider
from insights.core.exceptions import SkipComponent
from insights.core.plugins import component


@component(CloudProvider)
class IsAWS(object):
    """
    This component uses ``CloudProvider`` combiner to determine the cloud
    provider of the instance.
    It checks if AWS, if not AWS it raises ``SkipComponent``.

    Raises:
        SkipComponent: When it's not an instance from AWS.
    """
    def __init__(self, cp):
        if not cp or cp.cloud_provider != CloudProvider.AWS:
            raise SkipComponent("Not AWS instance")


@component(CloudProvider)
class IsAzure(object):
    """
    This component uses ``CloudProvider`` combiner to determine the cloud
    provider of the instance.
    It checks if Azure, if not Azure it raises ``SkipComponent``.

    Raises:
        SkipComponent: When it's not an instance from Azure.
    """
    def __init__(self, cp):
        if not cp or cp.cloud_provider != CloudProvider.AZURE:
            raise SkipComponent("Not Azure instance")


@component(CloudProvider)
class IsGCP(object):
    """
    This component uses ``CloudProvider`` combiner to determine the cloud
    provider of the instance.
    It checks if Google Cloud Platform (GCP), if not GCP it raises ``SkipComponent``.

    Raises:
        SkipComponent: When it's not an instance from GCP.
    """
    def __init__(self, cp):
        if not cp or cp.cloud_provider != CloudProvider.GOOGLE:
            raise SkipComponent("Not Google Cloud Platform instance")



"""
Microsoft SQL Server Database Engine configuration - file ``/var/opt/mssql/mssql.conf``
=======================================================================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.mssql_conf)
class MsSQLConf(IniConfigFile):
    """
    The Microsoft SQL Server configuration file is a standard '.ini' file and uses
    the ``IniConfigfile`` class to read it.

    Sample configuration::

        [sqlagent]
        enabled = false

        [EULA]
        accepteula = Y

        [memory]
        memorylimitmb = 3328

    Examples:
        >>> type(conf)
        <class 'insights.parsers.mssql_conf.MsSQLConf'>
        >>> conf.has_option('memory', 'memorylimitmb')
        True
        >>> conf.get('memory', 'memorylimitmb') == '3328'
        True
    """
    pass



"""
SELinuxEnabled
==============

The ``SELinuxEnabled`` component uses ``SelinuxConfig`` parser to determine if
SELinux is enabled or not.  The ``SkipComponent`` will be raised only when
"SELINUX=disabled" is set.
"""
from insights.core.exceptions import SkipComponent
from insights.core.plugins import component
from insights.parsers.selinux_config import SelinuxConfig


@component(SelinuxConfig)
class SELinuxEnabled(object):
    """
    Raises:
        SkipComponent: When ``SELINUX=disabled`` is set.
    """
    def __init__(self, seconf):
        self.selinux = seconf.get('SELINUX')
        if self.selinux == 'disabled':
            raise SkipComponent("SELinux not enabled")



"""
Kernel system control information
=================================

Shared parsers for parsing file ``/etc/sysctl.conf`` and command ``sysctl -a``.

Parsers included in this module are:

Sysctl - command ``sysctl -a``
------------------------------

SysctlConf - file ``/etc/sysctl.conf``
--------------------------------------

SysctlDConfEtc - file ``/etc/sysctl.d/*.conf``
----------------------------------------------

SysctlDConfUsr - file ``/usr/lib/sysctl.d/*.conf``
--------------------------------------------------

SysctlConfInitramfs - command ``lsinitrd``
------------------------------------------
"""
from collections import OrderedDict

from insights.core import CommandParser, TextFileOutput, Parser
from insights.core.plugins import parser
from insights.parsers import split_kv_pairs
from insights.specs import Specs


class SysctlBase(Parser, dict):
    """
    Parse sysctl conf files, and sysctl command output.

    Sample input::

        # sysctl.conf sample
        #
          kernel.domainname = example.com

        ; this one has a space which will be written to the sysctl!
          kernel.modprobe = /sbin/mod probe

    Attributes:
        data (OrderedDict): Dictionary containing key/value pairs for the lines in the
            configuration file.  Dictionary is in order keywords first
            appear in the lines.

    Examples:
        >>> sysctl_conf.data['kernel.domainname']
        'example.com'
        >>> sysctl_conf.data['kernel.modprobe']
        '/sbin/mod probe'
    """
    def parse_content(self, content):
        # Valid comments are both # and ; so remove one locally,
        # other comments and blank lines are removed by split_kv_pairs.
        lines = [l for l in content if not l.strip().startswith(';')]
        self.update(split_kv_pairs(lines, ordered=True))

    @property
    def data(self):
        return OrderedDict(self)


@parser(Specs.sysctl_conf)
class SysctlConf(SysctlBase):
    """
    Parse `/etc/sysctl.conf` file.

    .. note::
        Please refer to its base class :class:`SysctlBase`
        for the sample data and examples.
    """
    pass


@parser(Specs.sysctl_d_conf_etc)
class SysctlDConfEtc(SysctlBase):
    """
    Parse `/etc/sysctl.d/*.conf` files.

    .. note::
        Please refer to its base class :class:`SysctlBase`
        for the sample data and examples.
    """
    pass


@parser(Specs.sysctl_d_conf_usr)
class SysctlDConfUsr(SysctlBase):
    """
    Parse `/usr/lib/sysctl.d/*.conf` files.

    .. note::
        Please refer to its base class :class:`SysctlBase`
        for the sample data and examples.
    """
    pass


@parser(Specs.sysctl)
class Sysctl(SysctlBase):
    """
    Parse the output of `sysctl -a` command.

    .. note::
        Please refer to its base class :class:`SysctlBase`
        for the sample data and examples.
    """
    pass


@parser(Specs.sysctl_conf_initramfs)
class SysctlConfInitramfs(CommandParser, TextFileOutput):
    """Shared parser for the output of ``lsinitrd`` applied to kdump
    initramfs images to view ``sysctl.conf`` and ``sysctl.d``
    configurations.

    The file is treated as raw lines (as a ``TextFileOutput`` parser, as no
    serialial parsing result is required for this spec for now.

    .. note::
        Please refer to its super-class :class:`insights.core.TextFileOutput`

    Sample input::

        initramfs:/etc/sysctl.conf
        ========================================================================
        # sysctl settings are defined through files in
        # /usr/lib/sysctl.d/, /run/sysctl.d/, and /etc/sysctl.d/.
        #
        # Vendors settings live in /usr/lib/sysctl.d/.
        # To override a whole file, create a new file with the same in
        # /etc/sysctl.d/ and put new settings there. To override
        # only specific settings, add a file with a lexically later
        # name in /etc/sysctl.d/ and put new settings there.
        #
        # For more information, see sysctl.conf(5) and sysctl.d(5).
        fs.inotify.max_user_watches=524288
        ========================================================================

        initramfs:/etc/sysctl.d/*.conf
        ========================================================================

    Examples:
        >>> type(sysctl_initramfs)
        <class 'insights.parsers.sysctl.SysctlConfInitramfs'>
        >>> sysctl_initramfs.get('max_user_watches')[0]['raw_line']
        'fs.inotify.max_user_watches=524288'
    """
    def parse_content(self, content):
        # Remove all blank lines and comment lines prior to parsing
        valid_lines = []
        for line in content:
            line = line.strip()
            if line and not (line.startswith('#') or line.startswith(';')):
                valid_lines.append(line)
        super(SysctlConfInitramfs, self).parse_content(valid_lines)



import doctest
import pytest

from insights.core.exceptions import ParseException
from insights.parsers import sys_kernel
from insights.parsers.sys_kernel import SchedFeatures, SchedRTRuntime
from insights.tests import context_wrap

SYS_KERNEL_RUNTIME_CONTENT_1 = """
-1
""".strip()

SYS_KERNEL_RUNTIME_CONTENT_2 = """
950000
""".strip()

SYS_KERNEL_RUNTIME_CONTENT_3 = """
950000
-1
""".strip()

SYS_KERNEL_RUNTIME_CONTENT_4 = """
sss1
""".strip()

SYS_KERNEL_FEATURES = """
GENTLE_FAIR_SLEEPERS START_DEBIT NO_NEXT_BUDDY LAST_BUDDY CACHE_HOT_BUDDY
""".strip()


def test_sys_runtime_docs():
    failed, total = doctest.testmod(
        sys_kernel,
        globs={
            'srt': SchedRTRuntime(context_wrap(SYS_KERNEL_RUNTIME_CONTENT_2)),
            'sfs': SchedFeatures(context_wrap(SYS_KERNEL_FEATURES)),
        }
    )
    assert failed == 0


def test_sys_kernel_1():
    result = SchedRTRuntime(context_wrap(SYS_KERNEL_RUNTIME_CONTENT_1))
    assert result.runtime_us == -1


def test_exception():
    with pytest.raises(ParseException):
        SchedRTRuntime(context_wrap(SYS_KERNEL_RUNTIME_CONTENT_3))
    with pytest.raises(ParseException):
        SchedRTRuntime(context_wrap(SYS_KERNEL_RUNTIME_CONTENT_4))



"""
Custom datasources relate to Satellite Server
"""

import re

from insights.combiners.satellite_version import SatelliteVersion
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.specs import Specs

NODE_NOT_FOUND_ERROR = 'error Error on attach: Node not found'


class LocalSpecs(Specs):
    """Local specs used only by get_satellite_missed_pulp_agent_queues datasources"""

    content_host_uuids = simple_command(
        '/usr/bin/sudo -iu postgres /usr/bin/psql -d foreman -c "select uuid from katello_content_facets where uuid is not null;"',
        deps=[SatelliteVersion],
    )
    qpid_queues = simple_command(
        '/usr/bin/qpid-stat -q --ssl-certificate=/etc/pki/pulp/qpid/client.crt -b amqps://localhost:5671',
        deps=[SatelliteVersion],
    )


@datasource(
    LocalSpecs.content_host_uuids,
    Specs.messages,
    [LocalSpecs.qpid_queues, Specs.ls_la],
    HostContext,
    SatelliteVersion,
)
def satellite_missed_pulp_agent_queues(broker):
    """
    This datasource provides the missed pulp agent queues information on satellite server.

    Note:
        This datasource may be executed using the following command:

        ``insights cat --no-header satellite_missed_pulp_agent_queues``

    Sample output::

        pulp.agent.09008eec-aba6-4174-aa9f-e930004ce5c9:2018-01-16 00:06:13
        pulp.agent.fac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:16
        0

    Returns:
        str: All the missed pulp agent queues and the boolean mark if the data is
            truncated in the last line. If the value of last line is 0,
            it means all the missed queues are returned. If the value of the
            last line is 1, it means there are a lot of missed queues, to
            avoid render error, only the first 10 missed queues are returned.

    Raises:
        SkipComponent: When the error doen't happen or the missed queues have been recreated.

    """

    def _parse_non_existing_queues_in_msg():
        agentq_date_re = re.compile(
            r'^(?P<date>\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}) \[Protocol\] error Error on attach: Node not found: (?P<agentq>pulp.agent.[0-9a-f-]+)$'
        )
        agent_queue_last_date = {}
        ms_obj = broker[Specs.messages]
        for line in ms_obj.stream():
            if NODE_NOT_FOUND_ERROR in line:
                info, msg = [i.strip() for i in line.split(': ', 1)]
                info_splits = info.rsplit(None, 2)
                if len(info_splits) >= 3 and info_splits[2].startswith('qpidd'):
                    # The timestamp from syslog doesn't contain the year, but the
                    # message itself does - so use that.
                    match = agentq_date_re.search(msg)
                    if match:
                        agent_queue_last_date[match.group('agentq')] = match.group('date')
        return agent_queue_last_date

    def _get_content_host_uuid():
        output = broker[LocalSpecs.content_host_uuids].content
        host_uuids = []
        if len(output) > 3:
            for line in output[2:-1]:
                host_uuids.append(line.strip())
        return host_uuids

    def _get_qpid_queues():
        current_queues = []
        if LocalSpecs.qpid_queues in broker:
            try:
                output = broker[LocalSpecs.qpid_queues].content
                if len(output) > 3:
                    current_queues = [
                        line.split()[0].strip()
                        for line in output[3:]
                        if line.split()[0].startswith('pulp.agent')
                    ]
            except Exception:
                pass
        return current_queues

    def _get_qpid_jrn_dir_ouptput():
        current_queues = []
        if Specs.ls_la in broker:
            try:
                output = broker[Specs.ls_la].content
                if any('/var/lib/qpidd/qls/jrnl2' in line for line in output):
                    current_queues = [
                        line.split()[-1].strip()
                        for line in output
                        if line.split()[-1].startswith('pulp.agent')
                    ]
            except Exception:
                pass
        return current_queues

    missed_queues_in_log = _parse_non_existing_queues_in_msg()
    if missed_queues_in_log:
        host_uuids = _get_content_host_uuid()
        if host_uuids:
            qpid_output_queues = _get_qpid_queues()
            qpid_dir_queues = _get_qpid_jrn_dir_ouptput()
            if qpid_output_queues or qpid_dir_queues:
                qpid_queues = qpid_output_queues or qpid_dir_queues
                missed_queues = []
                too_more_data = 0
                for queue in missed_queues_in_log:
                    if queue.split('.')[-1] in host_uuids and queue not in qpid_queues:
                        missed_queues.append('%s:%s' % (queue, missed_queues_in_log[queue]))
                        # only return 10 missed queues in case too long data can't be rendered
                        if len(missed_queues) >= 10:
                            too_more_data = 1
                            break
                if missed_queues:
                    missed_queues.append(str(too_more_data))
                    return DatasourceProvider(
                        missed_queues,
                        relative_path='insights_datasources/satellite_missed_pulp_agent_queues',
                        ds=Specs.satellite_missed_pulp_agent_queues,
                        ctx=broker.get(HostContext),
                        cleaner=broker.get("cleaner"),
                    )
    raise SkipComponent



"""
Number of files or dirs from dirs
=================================

Parses the output of files or dirs number from dirs.
"""

import os

from insights.core import JSONParser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.files_dirs_number)
class FilesDirsNumberOfDir(JSONParser):
    """
    Get the files and dirs number from dirs.

    Sample output of this command is::

        {
          "/var/spool/postfix/maildrop/": {"dirs_number": 1, "files_number": 5},
          "/var/spool/clientmqueue/": {"dirs_number": 1, "files_number": 2},
        }

    Examples:
        >>> type(filesnumberofdir)
        <class 'insights.parsers.files_dirs_number_of_dirs.FilesDirsNumberOfDir'>
        >>> filesnumberofdir.files_number_of("/var/spool/postfix/maildrop")
        5
        >>> filesnumberofdir.dirs_number_of("/var/spool/clientmqueue/")
        1
    """

    def dirs_number_of(self, _dir):
        """Return the number of dirs under specified `_dir`, None when no such `_dir`"""
        _dir = os.path.join(_dir, '')  # ensure endswith '/'
        if _dir in self.data:
            return self.data[_dir]["dirs_number"]

    def files_number_of(self, _dir):
        """Return the number of files under specified `_dir`, None when no such `_dir`"""
        _dir = os.path.join(_dir, '')  # ensure endswith '/'
        if _dir in self.data:
            return self.data[_dir]["files_number"]



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import ceph_osd_tree_text
from insights.parsers.ceph_osd_tree_text import CephOsdTreeText
from insights.tests import context_wrap

OSD_TREE_CEPH_V3 = """
ID CLASS WEIGHT  TYPE NAME       STATUS REWEIGHT PRI-AFF
-1       0.08752 root default
-9       0.02917     host ceph1
 2   hdd 0.01459         osd.2       up  1.00000 1.00000
 5   hdd 0.01459         osd.5       up  1.00000 1.00000
-5       0.02917     host ceph2
 1   hdd 0.01459         osd.1       up  1.00000 1.00000
 4   hdd 0.01459         osd.4       up  1.00000 1.00000
-3       0.02917     host ceph3
 0   hdd 0.01459         osd.0       up  1.00000 1.00000
 3   hdd 0.01459         osd.3       up  1.00000 1.00000
-7             0     host ceph_1
""".strip()

OSD_TREE_CEPH_V2 = """
ID WEIGHT  TYPE NAME     UP/DOWN REWEIGHT PRIMARY-AFFINITY
-1 0.11670 root default
-2 0.02917     host osd1
 2 0.01459         osd.2      up  1.00000          1.00000
 5 0.01459         osd.5      up  1.00000          1.00000
-3 0.02917     host osd2
 1 0.01459         osd.1      up  1.00000          1.00000
 3 0.01459         osd.3      up  1.00000          1.00000
-4 0.02917     host osd3
 0 0.01459         osd.0      up  1.00000          1.00000
 4 0.01459         osd.4      up  1.00000          1.00000
-5 0.02917     host osd4
 6 0.01459         osd.6    down        0          1.00000
 7 0.01459         osd.7    down        0          1.00000
""".strip()

OSD_TREE_EMPTY = """
""".strip()

OSD_TREE_INVALID = """
ID WEIGHT  UP/DOWN REWEIGHT PRIMARY-AFFINITY
""".strip()


def test_ceph_osd_tree_text_v3():
    ceph_osd_tree = CephOsdTreeText(context_wrap(OSD_TREE_CEPH_V3))
    assert ceph_osd_tree['nodes'][0] == {'id': '-1', 'device_class': '', 'crush_weight': '0.08752', 'name': 'default',
                                         'status': '', 'reweight': '', 'primary_affinity': '', 'type': 'root',
                                         'children': [-7, -3, -5, -9]}


def test_ceph_osd_tree_text_v2():
    ceph_osd_tree = CephOsdTreeText(context_wrap(OSD_TREE_CEPH_V2))
    assert ceph_osd_tree['nodes'][1] == {'id': '-2', 'crush_weight': '0.02917', 'name': 'osd1', 'status': '',
                                         'reweight': '', 'primary_affinity': '', 'type': 'host', 'children': [5, 2]}


def test_skip_content():
    with pytest.raises(SkipComponent) as e:
        CephOsdTreeText(context_wrap(OSD_TREE_EMPTY))
    assert "Empty content." in str(e)


def test_error_content():
    with pytest.raises(ParseException) as e:
        CephOsdTreeText(context_wrap(OSD_TREE_INVALID))
    assert "Wrong content in table" in str(e)


def test_ceph_osd_tree_text_doc_examples():
    env = {
        'ceph_osd_tree_text': CephOsdTreeText(
            context_wrap(OSD_TREE_CEPH_V3)),
    }
    failed, total = doctest.testmod(ceph_osd_tree_text, globs=env)
    assert failed == 0



from insights.client.auto_config import try_auto_configuration
from insights.client.config import InsightsConfig
from insights.client.connection import InsightsConnection
from mock.mock import Mock
from mock.mock import patch
from pytest import mark


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_inventory_url_from_base_url(get_proxies, init_session):
    """
    Inventory URL is composed correctly from the given base URL.
    """
    config = Mock(base_url="www.example.com")
    connection = InsightsConnection(config)
    assert connection.inventory_url == "https://www.example.com/inventory/v1"


@mark.parametrize(("config_kwargs",), (({"check_results": True},), ({"checkin": True},)))
@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.auto_config._try_satellite6_configuration")
@patch('insights.client.config.argparse.ArgumentParser.parse_args')
def test_inventory_url_from_phase(
    _parse_args, try_satellite6_configuration, get_proxies, init_session, config_kwargs
):
    """
    Inventory URL is composed correctly from the default configuration.
    """
    config = InsightsConfig(**config_kwargs)
    config.load_all()  # Disables legacy upload.
    try_auto_configuration(config)  # Updates base_url if legacy upload is disabled.
    connection = InsightsConnection(config)
    assert (
        connection.inventory_url
        == "https://cert-api.access.redhat.com/r/insights/platform/inventory/v1"
    )



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import ip_netns_exec_namespace_lsof
from insights.parsers.ip_netns_exec_namespace_lsof import IpNetnsExecNamespaceLsofI
from insights.tests import context_wrap


IP_NETNS_EXEC_NAMESPACE_LSOF_I = """
COMMAND   PID   USER    FD  TYPE  DEVICE     SIZE/OFF  NODE NAME
neutron-n 975   root    5u  IPv4  6482691    0t0        TCP *:http (LISTEN)
""".strip()

EXCEPTION1 = """
""".strip()

EXCEPTION2 = """
COMMAND     PID   USER   FD   TYPE  DEVICE SIZE/OFF NODE NAME
""".strip()


def test_ip_netns_exec_namespace_lsof():
    data = IpNetnsExecNamespaceLsofI(context_wrap(IP_NETNS_EXEC_NAMESPACE_LSOF_I))
    assert len(data.search(node="TCP")) == 1
    assert len(data.search(command="neutron-n")) == 1
    assert len(data.search(user="nobody")) == 0
    assert data.data[0]["command"] == "neutron-n"
    assert data.data[0].get("node") == "TCP"
    assert [ps[2] for ps in data] == ["root"]


def test_ip_netns_exec_namespace_lsof_documentation():
    env = {
        "ns_lsof": IpNetnsExecNamespaceLsofI(context_wrap(IP_NETNS_EXEC_NAMESPACE_LSOF_I)),
    }
    failed, total = doctest.testmod(ip_netns_exec_namespace_lsof, globs=env)
    assert failed == 0


def test_ip_netns_exec_namespace_lsof_exception1():
    with pytest.raises(SkipComponent) as e:
        IpNetnsExecNamespaceLsofI(context_wrap(EXCEPTION1))
    assert "Empty file" in str(e)


def test_ip_netns_exec_namespace_lsof_exception2():
    with pytest.raises(SkipComponent) as e:
        IpNetnsExecNamespaceLsofI(context_wrap(EXCEPTION2))
    assert "Useless data" in str(e)



"""
snmpd parser
============
Parsers provided by this module are:

TcpIpStats - file ``/proc/net/snmp``
------------------------------------
The ``TcpIpStats`` class implements the parsing of ``/proc/net/snmp``
file, which contains TCP/IP stats of individual layer.

TcpIpStatsIPV6 - file ``/proc/net/snmp6``
-----------------------------------------
The ``TcpIpStatsIPV6`` class implements the parsing of ``/proc/net/snmp6``
file, which contains TCP/IP stats of individual layer.


SnmpdConf - file ``/etc/snmp/snmpd.conf``
-----------------------------------------
The ``SnmpdConf`` class implements the parsing of ``/etc/snmp/snmpd.conf``
file, which is the configuration file for the Net-SNMP SNMP agent.
"""

from insights.core import LegacyItemAccess, Parser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.proc_snmp_ipv4)
class TcpIpStats(Parser, LegacyItemAccess):
    """
    Parser for ``/proc/net/snmp`` file.

    Sample input is provided in the *Examples*.

    Examples:

        >>> SNMP_CONTENT = '''
        ... Ip: Forwarding DefaultTTL InReceives InHdrErrors InAddrErrors ForwDatagrams InUnknownProtos InDiscards InDelivers OutRequests OutDiscards OutNoRoutes ReasmTimeout ReasmReqds ReasmOKs ReasmFails FragOKs FragFails FragCreates
        ... Ip: 2 64 43767 0 0 0 0 0 41807 18407 12 73 0 0 0 10 0 0 0
        ... Icmp: InMsgs InErrors InCsumErrors InDestUnreachs InTimeExcds InParmProbs InSrcQuenchs InRedirects InEchos InEchoReps InTimestamps InTimestampReps InAddrMasks InAddrMaskReps OutMsgs OutErrors OutDestUnreachs OutTimeExcds OutParmProbs OutSrcQuenchs OutRedirects OutEchos OutEchoReps OutTimestamps OutTimestampReps OutAddrMasks OutAddrMaskReps
        ... Icmp: 34 0 0 34 0 0 0 0 0 0 0 0 0 0 44 0 44 0 0 0 0 0 0 0 0 0 0
        ... IcmpMsg: InType3 OutType3
        ... IcmpMsg: 34 44
        ... Tcp: RtoAlgorithm RtoMin RtoMax MaxConn ActiveOpens PassiveOpens AttemptFails EstabResets CurrEstab InSegs OutSegs RetransSegs InErrs OutRsts InCsumErrors
        ... Tcp: 1 200 120000 -1 444 0 0 6 7 19269 17050 5 4 234 0
        ... Udp: InDatagrams NoPorts InErrors OutDatagrams RcvbufErrors SndbufErrors InCsumErrors IgnoredMulti
        ... Udp: 18905 34 0 1348 0 0 0 3565
        ... UdpLite: InDatagrams NoPorts InErrors OutDatagrams RcvbufErrors SndbufErrors InCsumErrors IgnoredMulti
        ... UdpLite: 0 0 0 0 0 0 0 0
        ... '''.strip()
        >>> type(proc_snmp_ipv4)
        <class 'insights.parsers.snmp.TcpIpStats'>
        >>> snmp_stats = proc_snmp_ipv4.get("Ip")
        >>> snmp_stats["DefaultTTL"]
        64
        >>> snmp_stats = proc_snmp_ipv4.get("Udp")
        >>> snmp_stats["InDatagrams"]
        18905


    Resultant Data::

        {
            'Ip':
                {
                    'FragCreates': 0,
                    'ReasmFails': 10,
                    'Forwarding': 2,
                    'ReasmOKs': 0,
                    'ReasmReqds': 0,
                    'ReasmTimeout': 0,
                    ...
                    ...
                },
            'Icmp':
                {
                    'InRedirects': 0,
                    'InMsgs': 34,
                    'InSrcQuenchs': 0,
                    ...
                    ...
                }
            ...
            ...
        }
    """
    def parse_content(self, content):
        snmp_stats = {}
        data_id = []
        for line in content:
            line_split = line.split()
            if line_split[1].isdigit():
                snmp_stats[line_split[0].replace(":", "")] = dict(zip(data_id, map(int, line_split[1:])))
            else:
                data_id = line_split[1:]
        self.data = snmp_stats


@parser(Specs.proc_snmp_ipv6)
class TcpIpStatsIPV6(Parser, LegacyItemAccess):
    """
    Parser for ``/proc/net/snmp6`` file.

    Sample input is provided in the *Examples*.

    Examples:

        >>> SNMP_CONTENT = '''
        ... Ip6InReceives                   	757
        ... Ip6InHdrErrors                  	0
        ... Ip6InTooBigErrors               	0
        ... Ip6InNoRoutes                   	0
        ... Ip6InAddrErrors                 	0
        ... Ip6InDiscards                       10
        ... Ip6OutForwDatagrams             	0
        ... Ip6OutDiscards                  	0
        ... Ip6OutNoRoutes                  	0
        ... Ip6InOctets                     	579410
        ... Icmp6OutErrors                  	0
        ... Icmp6InCsumErrors               	0
        ... '''.strip()
        >>> type(proc_snmp_ipv6)
        <class 'insights.parsers.snmp.TcpIpStatsIPV6'>
        >>> IP6_RX_stats = proc_snmp_ipv6.get("Ip6InReceives")
        >>> IP6_RX_stats
        757
        >>> IP6_In_Disc = proc_snmp_ipv6.get("Ip6InDiscards")
        >>> IP6_In_Disc
        10


    Resultant Data::

        {
            'Ip6InReceives': 757,
            'Ip6InHdrErrors': 0,
            'Ip6InTooBigErrors': 0,
            'Ip6InNoRoutes': 0,
            'Ip6InAddrErrors': 0,
            'Ip6InDiscards': 10,
            ...
            ...
        }
    """

    def parse_content(self, content):
        snmp6_stats = {}
        for line in content:
            line_split = line.split()
            snmp6_stats[line_split[0]] = int(line_split[1]) if len(line_split) > 1 and line_split[1] else None
        self.data = snmp6_stats


@parser(Specs.snmpd_conf)
class SnmpdConf(Parser, dict):
    """
    Class for parsing the file ``/etc/snmp/snmpd.conf``

    Sample file content::

        #       sec.name  source          community
        com2sec notConfigUser  default       public

        #       groupName      securityModel securityName
        group   notConfigGroup v1           notConfigUser
        group   notConfigGroup v2c           notConfigUser

        # Make at least  snmpwalk -v 1 localhost -c public system fast again.
        #       name           incl/excl     subtree         mask(optional)
        view    systemview    included   .1.3.6.1.2.1.1
        view    systemview    included   .1.3.6.1.2.1.25.1.1

        #       group          context sec.model sec.level prefix read   write  notif
        access  notConfigGroup ""      any       noauth    exact  systemview none none

        dontLogTCPWrappersConnects yes
        include_ifmib_iface_prefix eth enp1s0

    Examples:
        >>> type(snmpd_conf)
        <class 'insights.parsers.snmp.SnmpdConf'>
        >>> snmpd_conf['dontLogTCPWrappersConnects']
        ['yes']
        >>> snmpd_conf['include_ifmib_iface_prefix']
        ['eth enp1s0']
    """

    def parse_content(self, content):
        content = get_active_lines(content)
        if not content:
            raise ParseException('Empty Content')

        for line in content:
            parts = line.split(None, 1)
            key = parts[0].strip()
            self.setdefault(key, [])
            if len(parts) > 1:
                value = parts[1].strip()
                self[key].append(value)



import logging
import pytest
import requests
from mock import mock

from insights.client.config import InsightsConfig
from insights.client.connection import InsightsConnection


class UnexpectedException(Exception):
    pass


LOGGING_FILE = "logging-file.log"
UPLOAD_URL = "https://www.example.com/insights"
LEGACY_URL = "https://www.example.com"
LEGACY_URL_SUFFIXES = ["/insights/", "", "/r", "/r/insights"]

EXCEPTION_MESSAGE = "request exception"
EXPECTED_EXCEPTIONS = [
    requests.exceptions.SSLError,
    requests.exceptions.ConnectTimeout,
    requests.exceptions.ReadTimeout,
]

parametrize_exceptions = pytest.mark.parametrize(
    ["exception"],
    [(exception,) for exception in EXPECTED_EXCEPTIONS],
)
parametrize_methods = pytest.mark.parametrize(
    ["http_method", "request_function"],
    [
        ("GET", "insights.client.connection.InsightsConnection.get"),
        ("POST", "insights.client.connection.InsightsConnection.post"),
    ],
)


@pytest.fixture
@mock.patch("insights.client.connection.InsightsConnection._init_session")
@mock.patch("insights.client.connection.InsightsConnection.get_proxies")
def insights_connection(get_proxies, init_session):
    config = InsightsConfig(
        base_url="www.example.com",
        logging_file=LOGGING_FILE,
        upload_url=UPLOAD_URL,
    )

    connection = InsightsConnection(config)
    connection.proxies = None

    return connection


@parametrize_methods
@mock.patch("insights.client.connection.InsightsConnection._legacy_test_urls")
def test_test_urls_legacy_test_urls_call(
    legacy_test_urls, http_method, request_function, insights_connection
):
    """The non-legacy URL subtest hands over to its legacy counterpart if legacy_upload is enabled."""
    insights_connection.config.legacy_upload = True

    insights_connection._test_urls(UPLOAD_URL, http_method)

    legacy_test_urls.assert_called_once_with(UPLOAD_URL, http_method)


@parametrize_methods
@mock.patch("insights.client.connection.InsightsConnection._legacy_test_urls")
def test_test_urls_legacy_test_urls_result(
    legacy_test_urls, http_method, request_function, insights_connection
):
    """The non-legacy URL subtest returns from its legacy counterpart if legacy_upload is enabled."""
    insights_connection.config.legacy_upload = True

    result = insights_connection._test_urls(UPLOAD_URL, http_method)

    assert result == legacy_test_urls.return_value


@parametrize_methods
@mock.patch("insights.client.connection.TemporaryFile")
@mock.patch("insights.client.connection.InsightsConnection._legacy_test_urls")
def test_test_urls_not_legacy_test_urls(
    legacy_test_urls, temporary_file, http_method, request_function, insights_connection
):
    """The non-legacy URL subtest doesn't hand over to its legacy counterpart if legacy_upload is disabled."""
    insights_connection.config.legacy_upload = False

    with mock.patch(request_function):
        insights_connection._test_urls(UPLOAD_URL, http_method)
    legacy_test_urls.assert_not_called()


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_test_urls_get_no_catch(get, temporary_file, post, insights_connection):
    """The non-legacy URL subtest issues one GET request in case of an unknown exception."""
    get.side_effect = UnexpectedException
    insights_connection.config.legacy_upload = False

    try:
        insights_connection._test_urls(UPLOAD_URL, "GET")
    except UnexpectedException:
        pass

    get.assert_called_once_with(UPLOAD_URL)
    post.assert_not_called()


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_test_urls_get_success(get, temporary_file, post, insights_connection):
    """The non-legacy URL subtest issues one GET request if the API call succeeds."""
    insights_connection.config.legacy_upload = False

    insights_connection._test_urls(UPLOAD_URL, "GET")

    get.assert_called_once_with(UPLOAD_URL)
    post.assert_not_called()


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_test_urls_get_fail(get, temporary_file, post, insights_connection, exception):
    """The non-legacy URL subtest issues one GET request if the API call fails."""
    get.side_effect = exception
    insights_connection.config.legacy_upload = False

    try:
        insights_connection._test_urls(UPLOAD_URL, "GET")
    except exception:
        pass

    get.assert_called_once_with(UPLOAD_URL)
    post.assert_not_called()


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_test_urls_post_no_catch(get, temporary_file, post, insights_connection):
    """The non-legacy URL subtest issues one POST request in case of an unknown exception."""
    post.side_effect = UnexpectedException
    insights_connection.config.legacy_upload = False

    try:
        insights_connection._test_urls(UPLOAD_URL, "POST")
    except UnexpectedException:
        pass

    post.assert_called_once_with(UPLOAD_URL, files=mock.ANY)
    get.assert_not_called()


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_test_urls_post_success(get, temporary_file, post, insights_connection):
    """The non-legacy URL subtest issues one POST request if the API call succeeds."""
    insights_connection.config.legacy_upload = False

    insights_connection._test_urls(UPLOAD_URL, "POST")
    post.assert_called_once_with(UPLOAD_URL, files=mock.ANY)
    get.assert_not_called()


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_test_urls_post_fail(get, temporary_file, post, exception, insights_connection):
    """The non-legacy URL subtest issues one POST request if the API call fails."""
    post.side_effect = exception
    insights_connection.config.legacy_upload = False

    try:
        insights_connection._test_urls(UPLOAD_URL, "POST")
    except exception:
        pass

    post.assert_called_once_with(UPLOAD_URL, files=mock.ANY)
    get.assert_not_called()


@parametrize_methods
@mock.patch("insights.client.connection.TemporaryFile")
def test_test_urls_no_catch(
    temporary_file, http_method, request_function, insights_connection
):
    """The non-legacy URL subtest doesn't catch unknown exceptions."""
    insights_connection.config.legacy_upload = False

    raised = UnexpectedException()

    with pytest.raises(UnexpectedException) as caught:
        with mock.patch(request_function) as request_function_mock:
            request_function_mock.side_effect = raised
            insights_connection._test_urls(UPLOAD_URL, http_method)

    assert caught.value is raised


@parametrize_exceptions
@parametrize_methods
@mock.patch("insights.client.connection.TemporaryFile")
def test_test_urls_raise(
    temporary_file, http_method, request_function, exception, insights_connection
):
    """The non-legacy URL subtest re-raises the API call failure exception."""
    insights_connection.config.legacy_upload = False

    raised = exception()

    with pytest.raises(exception) as caught:
        with mock.patch(request_function) as request_function_mock:
            request_function_mock.side_effect = raised
            insights_connection._test_urls(UPLOAD_URL, http_method)

    assert caught.value is raised


@parametrize_methods
def test_test_urls_error_log_no_catch(
    http_method, request_function, insights_connection, caplog
):
    """The non-legacy URL subtest doesn't log any errors in case of an unknown exception."""
    insights_connection.config.legacy_upload = False

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        with mock.patch(request_function) as request_function_mock:
            request_function_mock.side_effect = UnexpectedException
            try:
                insights_connection._test_urls(UPLOAD_URL, http_method)
            except UnexpectedException:
                pass

    assert not caplog.record_tuples


@parametrize_methods
def test_test_urls_error_log_success(
    http_method, request_function, insights_connection, caplog
):
    """The non-legacy URL subtest doesn't log any errors if all API calls succeed."""
    insights_connection.config.legacy_upload = False

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        with mock.patch(request_function):
            insights_connection._test_urls(UPLOAD_URL, http_method)

    assert not caplog.record_tuples


@parametrize_exceptions
@parametrize_methods
def test_test_urls_error_log_fail(
    http_method, request_function, exception, insights_connection, caplog
):
    """The non-legacy URL subtest logs an ERROR if an API call fails."""
    insights_connection.config.legacy_upload = False

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        try:
            with mock.patch(request_function) as request_function_mock:
                request_function_mock.side_effect = exception
                insights_connection._test_urls(UPLOAD_URL, http_method)
        except exception:
            pass

    assert caplog.record_tuples == [
        (
            "insights.client.connection",
            logging.ERROR,
            "Could not successfully connect to: {0}".format(UPLOAD_URL),
        )
    ]


@parametrize_methods
def test_test_urls_print_no_catch(
    http_method, request_function, insights_connection, capsys
):
    """The non-legacy URL subtest doesn't print anything in case of an unknown exception."""
    insights_connection.config.legacy_upload = False

    with mock.patch(request_function) as request_function_mock:
        request_function_mock.side_effect = UnexpectedException
        try:
            insights_connection._test_urls(UPLOAD_URL, http_method)
        except UnexpectedException:
            pass

    out, err = capsys.readouterr()
    assert not out
    assert not err


@parametrize_methods
def test_test_urls_print_success(
    http_method, request_function, insights_connection, capsys
):
    """The non-legacy URL subtest doesn't print anything if all API calls succeed."""
    insights_connection.config.legacy_upload = False

    with mock.patch(request_function):
        insights_connection._test_urls(UPLOAD_URL, http_method)

    out, err = capsys.readouterr()
    assert not out
    assert not err


@parametrize_exceptions
@parametrize_methods
def test_test_urls_print_fail(
    http_method, request_function, exception, insights_connection, capsys
):
    """The non-legacy URL subtest prints the exception details if an API call fails."""
    insights_connection.config.legacy_upload = False

    with mock.patch(request_function) as request_function_mock:
        request_function_mock.side_effect = exception(EXCEPTION_MESSAGE)
        try:
            insights_connection._test_urls(UPLOAD_URL, http_method)
        except exception:
            pass

    out, err = capsys.readouterr()
    assert out == "{0}\n".format(EXCEPTION_MESSAGE)
    assert not err


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_legacy_test_urls_get_no_catch(get, post, insights_connection):
    """The legacy URL subtest issues one GET request in case of an unknown exception."""
    get.side_effect = UnexpectedException

    try:
        insights_connection._legacy_test_urls(UPLOAD_URL, "GET")
    except UnexpectedException:
        pass

    get.assert_called_once_with(LEGACY_URL + LEGACY_URL_SUFFIXES[0])
    post.assert_not_called()


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_legacy_test_urls_get_success(get, post, insights_connection):
    """The non-legacy URL subtest issues one GET request if the API call succeeds."""
    insights_connection._legacy_test_urls(UPLOAD_URL, "GET")

    get.assert_called_once_with(LEGACY_URL + LEGACY_URL_SUFFIXES[0])
    post.assert_not_called()


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_legacy_test_urls_get_one_fail(get, post, exception, insights_connection):
    """The non-legacy URL subtest issues one GET request if one API call fails."""
    get.side_effect = [exception, mock.Mock()]

    insights_connection._legacy_test_urls(UPLOAD_URL, "GET")

    assert get.mock_calls == [
        mock.call(LEGACY_URL + LEGACY_URL_SUFFIXES[0]),
        mock.call(LEGACY_URL + LEGACY_URL_SUFFIXES[1]),
    ]
    post.assert_not_called()


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_legacy_test_urls_get_all_fails(get, post, exception, insights_connection):
    """The non-legacy URL subtest issues one GET request for every API call if the previous one fails."""
    get.side_effect = [exception] * len(LEGACY_URL_SUFFIXES)

    try:
        insights_connection._legacy_test_urls(UPLOAD_URL, "GET")
    except exception:
        pass

    assert get.mock_calls == [
        mock.call(LEGACY_URL + suffix) for suffix in LEGACY_URL_SUFFIXES
    ]
    post.assert_not_called()


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_legacy_test_urls_post_no_catch(get, post, insights_connection):
    """The legacy URL subtest issues one POST request in case of an unknown exception."""
    post.side_effect = UnexpectedException

    try:
        insights_connection._legacy_test_urls(UPLOAD_URL, "POST")
    except UnexpectedException:
        pass

    post.assert_called_once_with(LEGACY_URL + LEGACY_URL_SUFFIXES[0], data=mock.ANY)
    get.assert_not_called()


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_legacy_test_urls_post_success(get, post, insights_connection):
    """The non-legacy URL subtest issues one POST request if the API call succeeds."""
    insights_connection._legacy_test_urls(UPLOAD_URL, "POST")

    post.assert_called_once_with(LEGACY_URL + LEGACY_URL_SUFFIXES[0], data=mock.ANY)
    get.assert_not_called()


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_legacy_test_urls_post_one_fail(get, post, exception, insights_connection):
    """The non-legacy URL subtest issues one POST request if one API call fails."""
    post.side_effect = [exception, mock.Mock(status_code=200)]

    insights_connection._legacy_test_urls(UPLOAD_URL, "POST")

    assert post.mock_calls == [
        mock.call(LEGACY_URL + LEGACY_URL_SUFFIXES[0], data=mock.ANY),
        mock.call(LEGACY_URL + LEGACY_URL_SUFFIXES[1], data=mock.ANY),
    ]
    get.assert_not_called()


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.InsightsConnection.get")
def test_legacy_test_urls_post_all_fails(get, post, exception, insights_connection):
    """The non-legacy URL subtest issues one POST request for every API call if the previous one fails."""
    post.side_effect = [exception] * len(LEGACY_URL_SUFFIXES)

    try:
        insights_connection._legacy_test_urls(UPLOAD_URL, "POST")
    except exception:
        pass

    assert post.mock_calls == [
        mock.call(LEGACY_URL + suffix, data=mock.ANY) for suffix in LEGACY_URL_SUFFIXES
    ]
    get.assert_not_called()


@parametrize_methods
def test_legacy_test_urls_no_catch(http_method, request_function, insights_connection):
    """The legacy URL subtest doesn't catch unknown exceptions."""
    raised = UnexpectedException()

    with pytest.raises(UnexpectedException) as caught:
        with mock.patch(request_function) as request_function_mock:
            request_function_mock.side_effect = raised
            insights_connection._legacy_test_urls(UPLOAD_URL, http_method)

    assert caught.value is raised


@parametrize_exceptions
@parametrize_methods
def test_legacy_test_urls_raise(
    http_method, request_function, exception, insights_connection
):
    """The legacy URL subtest re-raises the last API call failure exception."""
    exceptions = [exception() for _ in LEGACY_URL_SUFFIXES]

    with pytest.raises(exception) as caught:
        with mock.patch(request_function) as request_function_mock:
            request_function_mock.side_effect = exceptions
            insights_connection._legacy_test_urls(UPLOAD_URL, http_method)

    assert caught.value is exceptions[-1]


@parametrize_methods
def test_legacy_test_urls_error_log_no_catch(
    http_method, request_function, insights_connection, caplog
):
    """The legacy URL subtest doesn't log any errors in case of an unknown exception."""
    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        with mock.patch(request_function) as request_function_mock:
            request_function_mock.side_effect = UnexpectedException
            try:
                insights_connection._legacy_test_urls(UPLOAD_URL, http_method)
            except UnexpectedException:
                pass

    assert not caplog.record_tuples


@parametrize_exceptions
@parametrize_methods
def test_legacy_test_urls_error_log_success(
    http_method, request_function, exception, insights_connection, caplog
):
    """The legacy URL subtest doesn't log any errors if the API call succeeds."""
    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        with mock.patch(request_function):
            insights_connection._legacy_test_urls(UPLOAD_URL, http_method)

    assert not caplog.record_tuples


@parametrize_exceptions
@parametrize_methods
def test_legacy_test_urls_error_log_one_fail(
    http_method, request_function, exception, insights_connection, caplog
):
    """The legacy URL subtest logs one ERROR if one API call fails."""
    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        try:
            with mock.patch(request_function) as request_function_mock:
                request_function_mock.side_effect = [exception(), mock.Mock()]
                insights_connection._legacy_test_urls(UPLOAD_URL, http_method)
        except exception:
            pass

    assert caplog.record_tuples == [
        (
            "insights.client.connection",
            logging.ERROR,
            "Could not successfully connect to: {0}".format(LEGACY_URL + LEGACY_URL_SUFFIXES[0]),
        )
    ]


@parametrize_exceptions
@parametrize_methods
def test_legacy_test_urls_error_log_all_fails(
    http_method, request_function, exception, insights_connection, caplog
):
    """The legacy URL subtest logs one ERROR for every failed API call."""
    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        try:
            with mock.patch(request_function) as request_function_mock:
                request_function_mock.side_effect = [
                    exception() for _ in LEGACY_URL_SUFFIXES
                ]
                insights_connection._legacy_test_urls(UPLOAD_URL, http_method)
        except exception:
            pass

    assert caplog.record_tuples == [
        (
            "insights.client.connection",
            logging.ERROR,
            "Could not successfully connect to: {0}{1}".format(LEGACY_URL, suffix),
        )
        for suffix in LEGACY_URL_SUFFIXES
    ]


@parametrize_methods
def test_legacy_test_urls_exception_print_no_catch(
    http_method, request_function, insights_connection, capsys
):
    """The legacy URL subtest doesn't print anything in case of an unknown exception."""
    with mock.patch(request_function) as request_function_mock:
        request_function_mock.side_effect = UnexpectedException
        try:
            insights_connection._legacy_test_urls(UPLOAD_URL, http_method)
        except UnexpectedException:
            pass

    out, err = capsys.readouterr()
    assert not out
    assert not err


@parametrize_exceptions
@parametrize_methods
def test_legacy_test_urls_exception_print_success(
    http_method, request_function, exception, insights_connection, capsys
):
    """The legacy URL subtest prints a message pointing to a log file if the API call succeeds."""
    with mock.patch(request_function):
        insights_connection._legacy_test_urls(UPLOAD_URL, http_method)

    out, err = capsys.readouterr()
    assert not out
    assert not err


@parametrize_exceptions
@parametrize_methods
def test_legacy_test_urls_exception_print_one_fail(
    http_method, request_function, exception, insights_connection, capsys
):
    """The legacy URL subtest prints the exception details if one API call fails."""
    try:
        with mock.patch(request_function) as request_function_mock:
            request_function_mock.side_effect = [
                exception(EXCEPTION_MESSAGE),
                mock.Mock(),
            ]
            insights_connection._legacy_test_urls(UPLOAD_URL, http_method)
    except exception:
        pass

    out, err = capsys.readouterr()
    assert out == "{0}\n".format(EXCEPTION_MESSAGE)
    assert not err


@parametrize_exceptions
@parametrize_methods
def test_legacy_test_urls_exception_print_all_fails(
    http_method, request_function, exception, insights_connection, capsys
):
    """The legacy connection test prints the details for every exception if all API calls fail."""
    exception_messages = [
        "{0} {1}".format(EXCEPTION_MESSAGE, suffix) for suffix in LEGACY_URL_SUFFIXES
    ]
    exceptions = [exception(message) for message in exception_messages]

    try:
        with mock.patch(request_function) as request_function_mock:
            request_function_mock.side_effect = exceptions
            insights_connection._legacy_test_urls(UPLOAD_URL, http_method)
    except exception:
        pass

    out, err = capsys.readouterr()
    assert out == "".join("{0}\n".format(message) for message in exception_messages)
    assert not err


@mock.patch("insights.client.connection.InsightsConnection._test_urls")
def test_test_connection_test_urls_no_catch(test_urls, insights_connection):
    """The connection test doesn't catch unknown exceptions."""
    test_urls.side_effect = UnexpectedException

    try:
        insights_connection.test_connection()
    except UnexpectedException:
        pass

    test_urls.assert_called_once()


@mock.patch("insights.client.connection.InsightsConnection._test_urls")
def test_test_connection_test_urls_success(test_urls, insights_connection):
    """The connection test performs several API calls in case of no error."""
    insights_connection.test_connection()
    assert len(test_urls.mock_calls) > 1


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection._test_urls")
def test_test_connection_test_urls_fail(test_urls, exception, insights_connection):
    """The connection test is stopped after the first API call failure."""
    test_urls.side_effect = exception

    insights_connection.test_connection()
    test_urls.assert_called_once()


@mock.patch("insights.client.connection.InsightsConnection._test_urls")
def test_test_connection_error_log_no_catch(test_urls, insights_connection, caplog):
    """The connection test doesn't log any ERROR in case of an unknown exception."""
    test_urls.side_effect = UnexpectedException

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        try:
            insights_connection.test_connection()
        except UnexpectedException:
            pass

    assert not caplog.record_tuples


@mock.patch("insights.client.connection.InsightsConnection._test_urls")
def test_test_connection_error_log_success(test_urls, insights_connection, caplog):
    """The connection test doesn't log any ERROR if all API calls succeed."""
    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        insights_connection.test_connection()

    assert not caplog.record_tuples


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection._test_urls")
def test_test_connection_error_log_fail(
    test_urls, exception, insights_connection, caplog
):
    """An error message is logged on the ERROR level."""
    test_urls.side_effect = exception

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        insights_connection.test_connection()

    assert caplog.record_tuples == [
        (
            "insights.client.connection",
            logging.ERROR,
            "Connectivity test failed! Please check your network configuration",
        )
    ]


@mock.patch("insights.client.connection.InsightsConnection._test_urls")
def test_test_connection_print_no_catch(test_urls, insights_connection, capsys):
    """The connection test doesn't print anything in case of an unknown exception."""
    test_urls.side_effect = UnexpectedException

    try:
        insights_connection.test_connection()
    except UnexpectedException:
        pass

    out, err = capsys.readouterr()
    assert not out
    assert not err


@mock.patch("insights.client.connection.InsightsConnection._test_urls")
def test_test_connection_print_success(test_urls, insights_connection, capsys):
    """The connection test prints a message pointing to a log file if all API calls succeed."""
    insights_connection.test_connection()

    out, err = capsys.readouterr()
    assert out == "See {0} for more details.\n".format(LOGGING_FILE)
    assert not err


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection._test_urls")
def test_test_connection_print_fail(test_urls, exception, insights_connection, capsys):
    """The connection test prints a message pointing to a log file if an API call fails."""
    test_urls.side_effect = exception("request exception")

    insights_connection.test_connection()
    out, err = capsys.readouterr()
    assert out == "Additional information may be in {0}\n".format(
        LOGGING_FILE,
    )
    assert not err


@mock.patch("insights.client.connection.InsightsConnection._test_urls")
def test_test_connection_no_catch(test_urls, insights_connection):
    """The connection test doesn't catch and recreate an unknown exception."""
    expected = UnexpectedException()
    test_urls.side_effect = expected

    with pytest.raises(UnexpectedException) as caught:
        insights_connection.test_connection()

    assert caught.value is expected


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
def test_test_connection_non_legacy_error_log_no_catch(
    temporary_file, post, insights_connection, caplog
):
    """The non-legacy connection test doesn't log any errors in case of an unknown exception."""
    post.side_effect = UnexpectedException
    insights_connection.config.legacy_upload = False

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        try:
            insights_connection.test_connection()
        except UnexpectedException:
            pass

    assert not caplog.record_tuples


@mock.patch("insights.client.connection.InsightsConnection.get")
@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
def test_test_connection_non_legacy_error_log_success(
    temporary_file, post, get, insights_connection, caplog
):
    """The non-legacy connection test doesn't log any errors if all API calls succeed."""
    insights_connection.config.legacy_upload = False

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        insights_connection.test_connection()

    assert not caplog.record_tuples


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
def test_test_connection_non_legacy_error_log_fail(
    temporary_file, post, exception, insights_connection, caplog
):
    """The connection test logs an ERROR if an API call fails."""
    post.side_effect = exception
    insights_connection.config.legacy_upload = False

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        insights_connection.test_connection()

    assert caplog.record_tuples == [
        ("insights.client.connection", logging.ERROR, message)
        for message in [
            "Could not successfully connect to: {0}".format(UPLOAD_URL),
            "Connectivity test failed! Please check your network configuration",
        ]
    ]


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
def test_test_connection_non_legacy_print_no_catch(
    temporary_file, post, insights_connection, capsys
):
    """The non-legacy connection test doesn't print anything in case of an unknown exception."""
    post.side_effect = UnexpectedException
    insights_connection.config.legacy_upload = False

    try:
        insights_connection.test_connection()
    except UnexpectedException:
        pass

    out, err = capsys.readouterr()
    assert not out
    assert not err


@mock.patch("insights.client.connection.InsightsConnection.get")
@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
def test_test_connection_non_legacy_print_success(
    temporary_file, post, get, insights_connection, capsys
):
    """The non-legacy connection test prints a message pointing to a log file if all API calls succeed."""
    insights_connection.config.legacy_upload = False

    insights_connection.test_connection()

    out, err = capsys.readouterr()
    assert out == "See {0} for more details.\n".format(LOGGING_FILE)
    assert not err


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
def test_test_connection_non_legacy_print_fail(
    temporary_file, post, exception, insights_connection, capsys
):
    """The non-legacy connection test prints the exception details and a message pointing to a log file if an API call fails."""
    post.side_effect = exception(EXCEPTION_MESSAGE)
    insights_connection.config.legacy_upload = False

    insights_connection.test_connection()

    out, err = capsys.readouterr()
    assert out == "{0}\nAdditional information may be in {1}\n".format(
        EXCEPTION_MESSAGE, LOGGING_FILE
    )
    assert not err


@mock.patch("insights.client.connection.InsightsConnection.post")
def test_test_connection_legacy_error_log_no_catch(post, insights_connection, caplog):
    """The legacy connection test doesn't log any errors in case of an unknown exception."""
    post.side_effect = UnexpectedException
    insights_connection.config.legacy_upload = True

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        try:
            insights_connection.test_connection()
        except UnexpectedException:
            pass

    assert not caplog.record_tuples


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
def test_test_connection_legacy_error_log_success(
    post, exception, insights_connection, caplog
):
    """The legacy connection test doesn't log any errors if all API calls succeed."""
    insights_connection.config.legacy_upload = True

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        insights_connection.test_connection()

    assert not caplog.record_tuples


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
def test_test_connection_legacy_error_log_one_fail_connection(
    post, exception, insights_connection, caplog
):
    """The connection test logs one ERROR if one API call fails."""
    post.side_effect = [exception, mock.Mock()]
    insights_connection.config.legacy_upload = True

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        insights_connection.test_connection()

    assert caplog.record_tuples == [
        (
            "insights.client.connection",
            logging.ERROR,
            "Could not successfully connect to: {0}".format(LEGACY_URL + LEGACY_URL_SUFFIXES[0])
        )
    ]


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
def test_test_connection_legacy_error_log_all_fails_connection(
    post, exception, insights_connection, caplog
):
    """The connection test logs one ERROR for every failed API call."""
    post.side_effect = exception
    insights_connection.config.legacy_upload = True

    with caplog.at_level(logging.ERROR, logger="insights.client.connection"):
        insights_connection.test_connection()

    legacy_test_url_errors = [
        "Could not successfully connect to: {0}{1}".format(LEGACY_URL, suffix)
        for suffix in LEGACY_URL_SUFFIXES
    ]
    test_connection_errors = [
        "Connectivity test failed! Please check your network configuration",
    ]
    assert caplog.record_tuples == [
        ("insights.client.connection", logging.ERROR, message)
        for message in legacy_test_url_errors + test_connection_errors
    ]


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
def test_test_connection_legacy_print_no_catch(
    temporary_file, post, insights_connection, capsys
):
    """The legacy connection test doesn't print anything in case of an unknown exception."""
    post.side_effect = UnexpectedException
    insights_connection.config.legacy_upload = False

    try:
        insights_connection.test_connection()
    except UnexpectedException:
        pass

    out, err = capsys.readouterr()
    assert not out
    assert not err


@mock.patch("insights.client.connection.InsightsConnection.post")
@mock.patch("insights.client.connection.TemporaryFile")
def test_test_connection_legacy_print_success(
    temporary_file, post, insights_connection, capsys
):
    """The legacy connection test prints a message pointing to a log file if all API calls succeed."""
    insights_connection.config.legacy_upload = False

    insights_connection.test_connection()

    out, err = capsys.readouterr()
    assert out == "See {0} for more details.\n".format(LOGGING_FILE)
    assert not err


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
def test_test_connection_legacy_print_one_fail(
    post, exception, insights_connection, capsys
):
    """The legacy connection test prints the exception details and a message pointing to a log file if one API call fails."""
    post.side_effect = [exception(EXCEPTION_MESSAGE), mock.Mock()]
    insights_connection.config.legacy_upload = True

    insights_connection.test_connection()

    out, err = capsys.readouterr()
    assert out == "{0}\nSee {1} for more details.\n".format(
        EXCEPTION_MESSAGE, LOGGING_FILE
    )
    assert not err


@parametrize_exceptions
@mock.patch("insights.client.connection.InsightsConnection.post")
def test_test_connection_legacy_print_all_fails(
    post, exception, insights_connection, capsys
):
    """The legacy connection test prints the details for every exception and a message pointing to a log file if all API calls fail."""
    exception_messages = [
        exception("{0} {1}".format(EXCEPTION_MESSAGE, suffix))
        for suffix in LEGACY_URL_SUFFIXES
    ]
    post.side_effect = [exception(message) for message in exception_messages]
    insights_connection.config.legacy_upload = True

    insights_connection.test_connection()

    out, err = capsys.readouterr()

    test_urls_messages = ["{0}\n".format(message) for message in exception_messages]
    test_connection_messages = [
        "Additional information may be in {0}\n".format(LOGGING_FILE)
    ]

    assert out == "".join(test_urls_messages + test_connection_messages)
    assert not err



import csv
import json
import os
import tempfile

from mock.mock import patch
from pytest import mark

from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.cleaner import Cleaner
from insights.cleaner.utilities import write_report

hostname = "report.test.com"
test_file_data = '{0}'.format(hostname)
test_file_data += 'testword\n'
test_file_data += 'ipv4: 10.0.2.155\n'
test_file_data += 'ipv6: abcd::1\n'
test_file_data += 'mac: 10:20:02:15:f5:ab'


@mark.parametrize(
    ("obfuscate", "obfuscate_hostname", "obfuscation_list", "keywords"),
    [
        (False, False, None, []),
        (True, False, None, []),
        (True, True, None, []),
        (True, True, None, ['testword']),
        (None, None, 'ipv4, mac', []),
        (None, None, 'ipv6, hostname', []),
        (None, None, 'ipv4, ipv6, hostname, mac', ['testword']),
    ],
)
@mark.parametrize("test_umask", [0o000, 0o022])
def test_rhsm_facts(test_umask, obfuscate, obfuscate_hostname, obfuscation_list, keywords):
    rhsm_facts_file = os.path.join(tempfile.gettempdir(), 'insights_test_rhsm.facts')
    conf = InsightsConfig(
        obfuscate=obfuscate,
        obfuscate_hostname=obfuscate_hostname,
        obfuscation_list=obfuscation_list,
    )
    conf.rhsm_facts_file = rhsm_facts_file
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    old_umask = os.umask(test_umask)
    pp = Cleaner(conf, {'keywords': keywords}, hostname)
    pp.clean_file(test_file, [])
    pp.generate_report(arch.archive_name)
    arch.delete_archive_dir()

    umask_after_test = os.umask(old_umask)

    assert os.path.isfile(rhsm_facts_file)
    st = os.stat(rhsm_facts_file)
    assert st.st_mode & 0o777 == 0o644 & ~test_umask
    assert umask_after_test == test_umask  # umask was not changed by Cleaner
    with open(rhsm_facts_file, 'r') as fp:
        facts = json.load(fp)
        # hostname
        assert facts['insights_client.hostname'] == hostname
        assert facts['insights_client.obfuscate_hostname_enabled'] == bool(
            obfuscate_hostname or obfuscation_list and 'hostname' in obfuscation_list
        )
        hns = json.loads(facts['insights_client.obfuscated_hostname'])
        if obfuscate_hostname or obfuscation_list and 'hostname' in obfuscation_list:
            assert hns[0]['original'] == hostname
            assert '.example.com' in hns[0]['obfuscated']
            assert len(hns[0]['obfuscated'].split('.', 1)[0]) == 12
        else:
            assert hns == []
        # ip
        assert facts['insights_client.obfuscate_ipv4_enabled'] == bool(
            obfuscate or obfuscation_list and 'ipv4' in obfuscation_list
        )
        ips = json.loads(facts['insights_client.obfuscated_ipv4'])
        if obfuscate or obfuscation_list and 'ipv4' in obfuscation_list:
            assert ips[0]['original'] == '10.0.2.155'
            assert ips[0]['obfuscated'] == '10.230.230.1'
        else:
            assert ips == []
        # ipv6
        assert facts['insights_client.obfuscate_ipv6_enabled'] == bool(
            obfuscation_list and 'ipv6' in obfuscation_list
        )
        ips = json.loads(facts['insights_client.obfuscated_ipv6'])
        if obfuscation_list and 'ipv6' in obfuscation_list:
            assert ips[0]['original'] == 'abcd::1'
            assert ips[0]['obfuscated'] != 'abcd::1'
            assert len(ips[0]['obfuscated']) == len('abcd::1')
        else:
            assert ips == []
        # mac
        assert facts['insights_client.obfuscate_mac_enabled'] == bool(
            obfuscation_list and 'mac' in obfuscation_list
        )
        macs = json.loads(facts['insights_client.obfuscated_mac'])
        if obfuscation_list and 'mac' in obfuscation_list:
            assert macs[0]['original'] == '10:20:02:15:f5:ab'
            assert macs[0]['obfuscated'] == 'b1:91:bc:f1:54:da'
        else:
            assert macs == []
        # keyword
        kws = json.loads(facts['insights_client.obfuscated_keyword'])
        if keywords:
            assert kws[0]['original'] == 'testword'
            assert kws[0]['obfuscated'] == 'keyword0'
        else:
            assert kws == []

    os.unlink(rhsm_facts_file)


@mark.parametrize(
    ("obfuscate", "obfuscate_hostname", "obfuscation_list", "keywords"),
    [
        (False, False, None, []),
        (True, False, None, []),
        (True, True, None, []),
        (True, True, None, ['testword']),
        (None, None, 'ipv4, mac', []),
        (None, None, 'ipv6, hostname', []),
        (None, None, 'ipv4, ipv6, hostname, mac', ['testword']),
    ],
)
@patch('insights.cleaner.Cleaner.generate_rhsm_facts', return_value=None)
def test_all_csv_reports(rhsm_facts, obfuscate, obfuscate_hostname, obfuscation_list, keywords):
    conf = InsightsConfig(
        obfuscate=obfuscate,
        obfuscate_hostname=obfuscate_hostname,
        obfuscation_list=obfuscation_list,
    )
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, {'keywords': keywords}, hostname)
    ip_report_file = os.path.join(pp.report_dir, "%s-ipv4.csv" % arch.archive_name)
    ipv6_report_file = os.path.join(pp.report_dir, "%s-ipv6.csv" % arch.archive_name)
    hn_report_file = os.path.join(pp.report_dir, "%s-hostname.csv" % arch.archive_name)
    kw_report_file = os.path.join(pp.report_dir, "%s-keyword.csv" % arch.archive_name)
    mac_report_file = os.path.join(pp.report_dir, "%s-mac.csv" % arch.archive_name)
    if os.path.isfile(ip_report_file):
        os.unlink(ip_report_file)
    if os.path.isfile(ipv6_report_file):
        os.unlink(ipv6_report_file)
    if os.path.isfile(hn_report_file):
        os.unlink(hn_report_file)
    if os.path.isfile(kw_report_file):
        os.unlink(kw_report_file)
    if os.path.isfile(mac_report_file):
        os.unlink(mac_report_file)

    pp.clean_file(test_file, [])
    pp.generate_report(arch.archive_name)
    arch.delete_archive_dir()

    if obfuscate or obfuscation_list and 'ipv4' in obfuscation_list:
        assert os.path.isfile(ip_report_file)
        with open(ip_report_file, 'r') as fp:
            ips = list(csv.reader(fp.readlines(), skipinitialspace=True))
            # ip
            assert len(ips) > 1
            assert ips[0] == ['Obfuscated IPv4', 'Original IPv4']
            assert ips[1] == ['10.230.230.1', '10.0.2.155']
        os.unlink(ip_report_file)
    else:
        assert not os.path.isfile(ip_report_file)
    if obfuscation_list and 'ipv6' in obfuscation_list:
        assert os.path.isfile(ipv6_report_file)
        with open(ipv6_report_file, 'r') as fp:
            ips = list(csv.reader(fp.readlines(), skipinitialspace=True))
            # ipv6
            assert len(ips) > 1
            assert ips[0] == ['Obfuscated IPv6', 'Original IPv6']
            assert ips[1][1] == 'abcd::1'
            assert ips[1][0] != ips[1][1]
            assert len(ips[1][0]) == len(ips[1][1])
        os.unlink(ipv6_report_file)
    else:
        assert not os.path.isfile(ipv6_report_file)

    if obfuscate_hostname or obfuscation_list and 'hostname' in obfuscation_list:
        assert os.path.isfile(hn_report_file)
        with open(hn_report_file, 'r') as fp:
            hns = list(csv.reader(fp.readlines(), skipinitialspace=True))
            # hn
            assert len(hns) > 1
            assert hns[0] == ['Obfuscated Hostname', 'Original Hostname']
            assert len(hns[1][0].split('.', 1)[0]) == 12
        os.unlink(hn_report_file)
    else:
        assert not os.path.isfile(hn_report_file)

    if obfuscation_list and 'mac' in obfuscation_list:
        assert os.path.isfile(mac_report_file)
        with open(mac_report_file, 'r') as fp:
            hns = list(csv.reader(fp.readlines(), skipinitialspace=True))
            # hn
            assert len(hns) > 1
            assert hns[0] == ['Obfuscated MAC', 'Original MAC']
            assert hns[1] == ['b1:91:bc:f1:54:da', '10:20:02:15:f5:ab']
        os.unlink(mac_report_file)
    else:
        assert not os.path.isfile(hn_report_file)

    if keywords:
        assert os.path.isfile(kw_report_file)
        with open(kw_report_file, 'r') as fp:
            kws = list(csv.reader(fp.readlines(), skipinitialspace=True))
            # kw
            assert len(kws) > 1
            assert kws[0] == ['Replaced Keyword', 'Original Keyword']
            assert kws[1] == ['testword', 'keyword0']
        os.unlink(kw_report_file)
    else:
        assert not os.path.isfile(kw_report_file)


def test_wirte_report_exp():
    report_file = os.path.join(tempfile.gettempdir(), '_test.csv')
    write_report(None, report_file)
    os.unlink(report_file)



"""
Custom datasource malware detection
===================================

Only when option `--collector malware-detection` is specified, the insights
collection will collect data for malware-detection.  When it is specified,
only the Insights Spec
:py:attr:`insights.specs.default.DefaultSpecs.malware_detection` will be
collected. To collect other specs within the collection of malware-detection,
add them to the "persist" section in the "malware_detection_manifest"
pre-defined in the :py:mod:`insights.specs.datasources.manifests`
"""

import json
import logging

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources.malware_detection import MalwareDetectionClient


logger = logging.getLogger(__name__)


# timeout=0 disables the datasource timeout alarm,
# allowing malware-detection to run for as long as necessary
@datasource(HostContext, timeout=0)
def malware_detection(broker):
    """
    Custom datasource to collects content for malware scanner if a scanner is present on the system
    """
    try:
        # Only run malware-detection if it was passed as an option to insights-client
        insights_config = broker.get('client_config')
        if not (
            insights_config
            and hasattr(insights_config, 'app')
            and insights_config.app == 'malware-detection'
        ):
            raise SkipComponent(
                "Only run malware-detection app when specifically requested via --collector option"
            )

        # Manifest of malware-detection is wrapped in the 'insights_config'
        # and was set in the `insights.client.config.InsightsConfig._imply_options`
        mdc = MalwareDetectionClient(insights_config)
        scan_results = mdc.run()
        if scan_results:
            # cleaner is not required for this datasource
            return [
                DatasourceProvider(
                    content=scan_results[0], relative_path="malware-detection-results.json"
                ),
                DatasourceProvider(
                    content=json.dumps(scan_results[1]),
                    relative_path="malware-detection-results-raw.json",
                ),
            ]
        else:
            raise SkipComponent("No scan results were produced")
    except SkipComponent as msg:
        raise SkipComponent("Skipping malware-detection app: {0}".format(str(msg)))
    except Exception as err:
        from traceback import format_exc

        err_msg = "Unexpected exception in malware-detection app: {0}".format(str(err))
        logger.error(err_msg)
        logger.debug(format_exc())
        raise SkipComponent(err_msg)



from insights.client.auto_config import set_auto_configuration, _try_satellite6_configuration, try_auto_configuration
from mock.mock import Mock, patch


@patch("insights.client.auto_config.rhsmCertificate", Mock())
@patch("insights.client.auto_config.open", Mock())
@patch("insights.client.auto_config._importInitConfig")
@patch("insights.client.auto_config.set_auto_configuration")
def test_rhsm_legacy_url(set_auto_configuration, initConfig):
    '''
    Ensure the correct host URL is selected for auto_config on a legacy RHSM upload
    '''
    initConfig().get.side_effect = ['subscription.rhsm.redhat.com', '443', '', '', '', '', '', '']
    config = Mock(base_url=None, upload_url=None, legacy_upload=True)
    _try_satellite6_configuration(config)
    set_auto_configuration.assert_called_with(config, 'cert-api.access.redhat.com', None, None, False, False, rhsm_no_proxy=None)


@patch("insights.client.auto_config.rhsmCertificate", Mock())
@patch("insights.client.auto_config.open", Mock())
@patch("insights.client.auto_config._importInitConfig")
@patch("insights.client.auto_config.set_auto_configuration")
def test_rhsm_platform_url(set_auto_configuration, initConfig):
    '''
    Ensure the correct host URL is selected for auto_config on a platform RHSM upload
    '''
    initConfig().get.side_effect = ['subscription.rhsm.redhat.com', '443', '', '', '', '', '', '']
    config = Mock(base_url=None, upload_url=None, legacy_upload=False)
    _try_satellite6_configuration(config)
    # set_auto_configuration.assert_called_with(config, 'console.redhat.com', None, None, False)
    set_auto_configuration.assert_called_with(config, 'cert-api.access.redhat.com', None, None, False, False, rhsm_no_proxy=None)


@patch("insights.client.auto_config.rhsmCertificate", Mock())
@patch("insights.client.auto_config.open", Mock())
@patch("insights.client.auto_config._importInitConfig")
@patch("insights.client.auto_config.set_auto_configuration")
def test_rhsm_stage_legacy_url(set_auto_configuration, initConfig):
    '''
    Ensure the correct host URL is selected for auto_config on a legacy staging RHSM upload

    This will still force legacy_upload=False as there is no classic staging env,
    so the result is the same as platform upload.

    '''
    initConfig().get.side_effect = ['subscription.rhsm.stage.redhat.com', '443', '', '', '', '', '', '']
    config = Mock(base_url=None, upload_url=None, legacy_upload=True)
    _try_satellite6_configuration(config)
    # config.legacy_upload is modified in the function
    config.legacy_upload = False
    set_auto_configuration.assert_called_with(config, 'cert.cloud.stage.redhat.com', None, None, False, True, rhsm_no_proxy=None)


@patch("insights.client.auto_config.rhsmCertificate", Mock())
@patch("insights.client.auto_config.open", Mock())
@patch("insights.client.auto_config._importInitConfig")
@patch("insights.client.auto_config.set_auto_configuration")
def test_rhsm_stage_platform_url(set_auto_configuration, initConfig):
    '''
    Ensure the correct host URL is selected for auto_config on a platform staging RHSM upload
    '''
    initConfig().get.side_effect = ['subscription.rhsm.stage.redhat.com', '443', '', '', '', '', '', '']
    config = Mock(base_url=None, upload_url=None, legacy_upload=False)
    _try_satellite6_configuration(config)
    set_auto_configuration.assert_called_with(config, 'cert.cloud.stage.redhat.com', None, None, False, True, rhsm_no_proxy=None)


@patch("insights.client.auto_config.rhsmCertificate", Mock())
@patch("insights.client.auto_config.open", Mock())
@patch("insights.client.auto_config._importInitConfig")
@patch("insights.client.auto_config.set_auto_configuration")
def test_sat_legacy_url(set_auto_configuration, initConfig):
    '''
    Ensure the correct host URL is selected for auto_config on a legacy Sat upload
    '''
    initConfig().get.side_effect = ['test.satellite.com', '443', '', '', '', '', '', 'test_cert']
    config = Mock(base_url=None, upload_url=None, legacy_upload=True)
    _try_satellite6_configuration(config)
    set_auto_configuration.assert_called_with(config, 'test.satellite.com:443/redhat_access', 'test_cert', None, True, False, rhsm_no_proxy=None)


@patch("insights.client.auto_config.rhsmCertificate", Mock())
@patch("insights.client.auto_config.open", Mock())
@patch("insights.client.auto_config._importInitConfig")
@patch("insights.client.auto_config.set_auto_configuration")
def test_sat_platform_url(set_auto_configuration, initConfig):
    '''
    Ensure the correct host URL is selected for auto_config on a platform Sat upload
    '''
    initConfig().get.side_effect = ['test.satellite.com', '443', '', '', '', '', '', 'test_cert']
    config = Mock(base_url=None, upload_url=None, legacy_upload=False)
    _try_satellite6_configuration(config)
    set_auto_configuration.assert_called_with(config, 'test.satellite.com:443/redhat_access', 'test_cert', None, True, False, rhsm_no_proxy=None)


@patch("insights.client.auto_config.verify_connectivity", Mock())
def test_rhsm_legacy_base_url_configured():
    '''
    Ensure the correct base URL is assembled for a legacy RHSM upload
    '''
    config = Mock(base_url=None, upload_url=None, legacy_upload=True, proxy=None)
    set_auto_configuration(config, 'cert-api.access.redhat.com', None, None, False, False)
    assert config.base_url == 'cert-api.access.redhat.com/r/insights'


@patch("insights.client.auto_config.verify_connectivity", Mock())
def test_rhsm_platform_base_url_configured():
    '''
    Ensure the correct base URL is assembled for a platform RHSM upload
    '''
    config = Mock(base_url=None, upload_url=None, legacy_upload=False, proxy=None)
    # set_auto_configuration(config, 'console.redhat.com', None, None, False)
    # assert config.base_url == 'console.redhat.com/api'
    # [CIRCUS MUSIC]
    set_auto_configuration(config, 'cert-api.access.redhat.com', None, None, False, False)
    # assert config.base_url == 'cert-api.access.redhat.com/r/insights/platform'
    assert config.base_url == 'cert-api.access.redhat.com/r/insights'


@patch("insights.client.auto_config.verify_connectivity", Mock())
def test_sat_legacy_base_url_configured():
    '''
    Ensure the correct base URL is assembled for a legacy RHSM upload
    '''
    config = Mock(base_url=None, upload_url=None, legacy_upload=True, proxy=None)
    set_auto_configuration(config, 'test.satellite.com:443/redhat_access', 'test_cert', None, True, False)
    assert config.base_url == 'test.satellite.com:443/redhat_access/r/insights'


@patch("insights.client.auto_config.verify_connectivity", Mock())
def test_sat_platform_base_url_configured():
    '''
    Ensure the correct base URL is assembled for a platform RHSM upload
    '''
    config = Mock(base_url=None, upload_url=None, legacy_upload=False, proxy=None)
    set_auto_configuration(config, 'test.satellite.com:443/redhat_access', 'test_cert', None, True, False)
    # assert config.base_url == 'test.satellite.com:443/redhat_access/r/insights/platform'
    assert config.base_url == 'test.satellite.com:443/redhat_access/r/insights'


@patch("insights.client.auto_config.verify_connectivity", Mock())
def test_platform_path_added():
    '''
    Ensure /platform is added when legacy_upload is false
    Ensure it's not added when legacy_upload is true
    '''
    # auto_config=True, legacy_upload=True
    config = Mock(base_url='test.satellite.com:443/redhat_access/r/insights', auto_config=True, legacy_upload=True, offline=False)
    try_auto_configuration(config)
    assert config.base_url == 'test.satellite.com:443/redhat_access/r/insights'

    # auto_config=True, legacy_upload=False
    config = Mock(base_url='test.satellite.com:443/redhat_access/r/insights', auto_config=True, legacy_upload=False, offline=False)
    try_auto_configuration(config)
    assert config.base_url == 'test.satellite.com:443/redhat_access/r/insights/platform'

    # auto_config=False, legacy_upload=True
    config = Mock(base_url='test.satellite.com:443/redhat_access/r/insights', auto_config=False, legacy_upload=True, offline=False)
    try_auto_configuration(config)
    assert config.base_url == 'test.satellite.com:443/redhat_access/r/insights'

    # auto_config=False, legacy_upload=False
    config = Mock(base_url='test.satellite.com:443/redhat_access/r/insights', auto_config=False, legacy_upload=False, offline=False)
    try_auto_configuration(config)
    assert config.base_url == 'test.satellite.com:443/redhat_access/r/insights/platform'


@patch("insights.client.auto_config._try_satellite6_configuration", Mock())
def test_platform_path_added_cloud_redhat():
    '''
    Ensure /platform is added when legacy_upload is false
    for any base_url ending in /r/insights, otherwise not added
    '''
    # classic API
    config = Mock(base_url='cert-api.access.redhat.com/r/insights', auto_config=True, legacy_upload=False, offline=False)
    try_auto_configuration(config)
    assert config.base_url == 'cert-api.access.redhat.com/r/insights/platform'

    # satellite
    config = Mock(base_url='test.satellite.com:443/redhat_access/r/insights', auto_config=True, legacy_upload=False, offline=False)
    try_auto_configuration(config)
    assert config.base_url == 'test.satellite.com:443/redhat_access/r/insights/platform'

    # console.redhat.com compatibility layer - classic API hosted on c.rh.c
    config = Mock(base_url='console.redhat.com/r/insights', auto_config=True, legacy_upload=False, offline=False)
    try_auto_configuration(config)
    assert config.base_url == 'console.redhat.com/r/insights/platform'

    # console.redhat.com API directly connected
    config = Mock(base_url='console.redhat.com/api', auto_config=True, legacy_upload=False, offline=False)
    try_auto_configuration(config)
    assert config.base_url == 'console.redhat.com/api'


@patch("insights.client.auto_config.verify_connectivity", Mock())
def test_rhsm_stage_legacy_base_url_configured():
    '''
    Ensure the correct base URL is assembled for a legacy staging RHSM upload

    This will still force legacy_upload=False as there is no classic staging env,
    so the result is the same as platform upload.
    '''
    config = Mock(base_url=None, upload_url=None, legacy_upload=True, proxy=None)
    set_auto_configuration(config, 'cert.cloud.stage.redhat.com', None, None, False, True)
    assert config.base_url == 'cert.cloud.stage.redhat.com/api'


@patch("insights.client.auto_config.verify_connectivity", Mock())
def test_rhsm_stage_platform_base_url_configured():
    '''
    Ensure the correct base URL is assembled for a platform staging RHSM upload
    '''
    config = Mock(base_url=None, upload_url=None, legacy_upload=False, proxy=None)
    set_auto_configuration(config, 'cert.cloud.stage.redhat.com', None, None, False, True)
    assert config.base_url == 'cert.cloud.stage.redhat.com/api'



from mock.mock import patch

from insights.client.config import InsightsConfig
from insights.cleaner import Cleaner


def test_obfuscate_hostname():
    hostname = 'test1.abc.com'
    line = "a line with %s here, test2.abc.com, test.redhat.com" % hostname
    c = InsightsConfig(obfuscate=True, obfuscate_hostname=True, hostname=hostname)
    pp = Cleaner(c, {}, hostname)
    actual = pp.clean_content(line)
    assert 'test1' not in actual
    assert 'test2' not in actual
    assert 'abc.com' not in actual
    assert len(actual.split('.example.com')[0].split()[-1]) == 12
    assert len(actual.split('.example.com')[1].split()[-1]) == 12
    assert '.example.com' in actual
    assert 'host' not in actual

    line = "a line w/o hostname, but test2.abc.com only"
    actual = pp.clean_content(line)
    assert 'test2' not in actual
    assert 'abc.com' not in actual
    assert '.example.com' in actual
    assert len(actual.split('.example.com')[0].split()[-1]) == 12

    hostname = 'test1'  # Short hostname
    line = "a line with %s here, test2.def.com" % hostname
    pp = Cleaner(c, {}, hostname)
    actual = pp.clean_content(line)
    assert hostname not in actual
    assert 'test2.def.com' in actual

    line = "a line w/o hostname"
    hostname = 'test1.abc.com'
    pp = Cleaner(c, {}, hostname)
    actual = pp.clean_content(line)
    assert line == actual


@patch("insights.cleaner.determine_hostname", return_value='test1.abc.com')
def test_obfuscate_hostname_determine_hostanme(hn):
    hostname = 'test1.abc.com'
    c = InsightsConfig(obfuscate=True, obfuscate_hostname=True, display_name='disp.abc.com')
    line = "a line with %s here, test2.def.com" % hostname
    pp = Cleaner(c, {})  # passed empty hostname to cleaner, determain it
    actual = pp.clean_content(line)
    assert hostname not in actual
    assert len(actual.split('.')[0].split()[-1]) == 12
    assert 'test2.def.com' in actual


@patch("insights.cleaner.determine_hostname", return_value='test1.abc.com')
def test_obfuscate_hostname_empty_line(hn):
    c = InsightsConfig(obfuscate=True, obfuscate_hostname=True, display_name='disp.abc.com')
    line = ""
    pp = Cleaner(c, {})  # passed empty hostname to cleaner, determain it
    actual = pp.clean_content(line)
    assert actual == line



from insights.parsers import candlepin_broker
from insights.parsers.candlepin_broker import CandlepinBrokerXML
from insights.tests import context_wrap
import doctest


CANDLEPIN_BROKER_1 = """
<configuration xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="urn:activemq /schema/artemis-configuration.xsd">

    <core xsi:schemaLocation="urn:activemq:core ">

        <security-enabled>true</security-enabled>

        <security-settings>
            <security-setting match="katello.candlepin.#">
                <permission roles="candlepinEventsConsumer" type="consume" />
            </security-setting>
            <security-setting match="#">
                <permission roles="invm-role" type="createAddress" />
                <permission roles="invm-role" type="deleteAddress" />
                <permission roles="invm-role" type="createDurableQueue" />
                <permission roles="invm-role" type="deleteDurableQueue" />
                <permission roles="invm-role" type="createNonDurableQueue" />
                <permission roles="invm-role" type="deleteNonDurableQueue" />
                <permission roles="invm-role" type="send" />
                <permission roles="invm-role" type="consume" />
                <permission roles="invm-role" type="browse" />
                <permission roles="invm-role" type="manage" />
            </security-setting>
        </security-settings>


        <cluster-user />
        <persistence-enabled>true</persistence-enabled>
        <journal-type>NIO</journal-type>
        <journal-pool-files>1</journal-pool-files>
        <create-bindings-dir>true</create-bindings-dir>
        <create-journal-dir>true</create-journal-dir>
        <bindings-directory>/var/lib/candlepin/activemq-artemis/bindings</bindings-directory>
        <journal-directory>/var/lib/candlepin/activemq-artemis/journal</journal-directory>
        <large-messages-directory>/var/lib/candlepin/activemq-artemis/largemsgs</large-messages-directory>
        <paging-directory>/var/lib/candlepin/activemq-artemis/paging</paging-directory>
        <max-disk-usage>99</max-disk-usage>
    <addresses>
            <address name="event.default">
                <multicast>
                    <queue name="event.org.candlepin.audit.LoggingListener" />
                    <queue name="event.org.candlepin.audit.ActivationListener" />
                </multicast>
            </address>

            <address name="katello.candlepin">
                <multicast>
                  <queue max-consumers="1" name="katello_candlepin_event_monitor.candlepin_events" />
                </multicast>
            </address>

            <address name="job">
                <multicast>
                    <queue name="jobs" />
                </multicast>
            </address>
        </addresses>

        <address-settings>
            <address-setting match="event.default">
                <config-delete-queues>FORCE</config-delete-queues>
                <auto-create-queues>true</auto-create-queues>
                <max-size-bytes>10485760</max-size-bytes>


                <page-size-bytes>1048576</page-size-bytes>


                <redelivery-delay>30000</redelivery-delay>
                <max-redelivery-delay>3600000</max-redelivery-delay>
                <redelivery-delay-multiplier>2</redelivery-delay-multiplier>
                <max-delivery-attempts>0</max-delivery-attempts>
            </address-setting>

            <address-setting match="katello.candlepin">
                <auto-create-queues>false</auto-create-queues>
                <max-size-bytes>10485760</max-size-bytes>


                <page-size-bytes>1048576</page-size-bytes>

                <redelivery-delay>0</redelivery-delay>
                <max-delivery-attempts>1</max-delivery-attempts>
            </address-setting>
            <address-setting match="job">
                <auto-create-queues>true</auto-create-queues>
                <max-size-bytes>10485760</max-size-bytes>


                <page-size-bytes>1048576</page-size-bytes>


                <redelivery-delay>30000</redelivery-delay>
                <max-redelivery-delay>3600000</max-redelivery-delay>
                <redelivery-delay-multiplier>2</redelivery-delay-multiplier>
                <max-delivery-attempts>0</max-delivery-attempts>
            </address-setting>
        </address-settings>

        <diverts>
            <divert name="katello_divert">
                <exclusive>false</exclusive>
                <address>event.default</address>
                <filter string="                     (EVENT_TARGET='COMPLIANCE' and EVENT_TYPE='CREATED') OR                     (EVENT_TARGET='SYSTEM_PURPOSE_COMPLIANCE' and EVENT_TYPE='CREATED') OR                     (EVENT_TARGET='POOL' and EVENT_TYPE='CREATED') OR                     (EVENT_TARGET='POOL' and EVENT_TYPE='DELETED') OR                     (EVENT_TARGET='ENTITLEMENT' and EVENT_TYPE='CREATED') OR                     (EVENT_TARGET='ENTITLEMENT' and EVENT_TYPE='DELETED') OR                     (EVENT_TARGET='OWNER_CONTENT_ACCESS_MODE' and EVENT_TYPE='MODIFIED')                 " />
                <forwarding-address>katello.candlepin</forwarding-address>
            </divert>
        </diverts>
    </core>
</configuration>
""".strip()


def test_candlepin_broker():
    broker = CandlepinBrokerXML(context_wrap(CANDLEPIN_BROKER_1))
    page_dirs = broker.get_elements('.//paging-directory')
    assert len(page_dirs) == 1
    assert page_dirs[0].text == '/var/lib/candlepin/activemq-artemis/paging'
    usage_eles = broker.get_elements('.//max-disk-usage')
    assert len(usage_eles) == 1
    assert usage_eles[0].text == '99'


def test_catalina_log_doc_examples():
    env = {
            'broker': CandlepinBrokerXML(context_wrap(CANDLEPIN_BROKER_1)),
          }
    failed, total = doctest.testmod(candlepin_broker, globs=env)
    assert failed == 0




import pytest
import doctest

from insights.core.exceptions import SkipComponent
from insights.parsers import ossl_files
from insights.tests import context_wrap

OSSL_FILES_CONFIG_CONTENT = """
/etc/pki/tls/openssl.cnf
""".strip()

OSSL_FILES_CONFIG_ERROR_2 = """
Error on line 11 of configuration file
00DE01245E7F0000:error:07000065:configuration file routines:def_load_bio:missing equal s    sign:crypto/conf/conf_def.c:525:HERE-->legacyline 11
""".strip()

OSSL_FILES_CONFIG_ERROR_3 = """

""".strip()

OSSL_FILES_CONFIG_CONTENT2 = """
         /etc/pki/tls/openssl.cnf
""".strip()


def test_ossl_files():
    ethtool_priv_flags = ossl_files.OsslFilesConfig(context_wrap(OSSL_FILES_CONFIG_CONTENT))
    assert ethtool_priv_flags.conf_path == "/etc/pki/tls/openssl.cnf"


def test_ossl_files_2():
    ethtool_priv_flags = ossl_files.OsslFilesConfig(context_wrap(OSSL_FILES_CONFIG_CONTENT2))
    assert ethtool_priv_flags.conf_path == "/etc/pki/tls/openssl.cnf"


def test_ossl_files_error():
    ethtool_priv_flags = ossl_files.OsslFilesConfig(context_wrap(OSSL_FILES_CONFIG_ERROR_2))
    assert ethtool_priv_flags.error_lines[0] == "Error on line 11 of configuration file"


def test_ossl_files_exception():
    with pytest.raises(SkipComponent):
        ossl_files.OsslFilesConfig(context_wrap(OSSL_FILES_CONFIG_ERROR_3))


def test_ethtool_i_doc_examples():
    env = {
        'openssl_cnf': ossl_files.OsslFilesConfig(context_wrap(OSSL_FILES_CONFIG_CONTENT)),
    }
    failed, total = doctest.testmod(ossl_files, globs=env)
    assert failed == 0



import doctest
from insights.parsers import scsi_eh_deadline
from insights.parsers.scsi_eh_deadline import SCSIEhDead
from insights.tests import context_wrap

SCSI_HOST0_PATH = "/sys/class/scsi_host/host0/eh_deadline"
SCSI_HOSTS_0 = """
off
""".strip()

SCSI_HOST1_PATH = "/sys/class/scsi_host/host1/eh_deadline"
SCSI_HOSTS_1 = """
10
""".strip()

SCSI_HOST2_PATH = "/sys/class/scsi_host/host2/eh_deadline"
SCSI_HOSTS_2 = """
-1
""".strip()


def test_scsi_eh_deadline_host0():
    context = context_wrap(SCSI_HOSTS_0, SCSI_HOST0_PATH)
    r = SCSIEhDead(context)
    assert r.scsi_host == 'host0'
    assert r.host_eh_deadline == 'off'
    assert r.data == {'host0': 'off'}


def test_scsi_eh_deadline_host1():
    context = context_wrap(SCSI_HOSTS_1, SCSI_HOST1_PATH)
    r = SCSIEhDead(context)
    assert r.scsi_host == 'host1'
    assert r.host_eh_deadline == '10'
    assert r.data == {'host1': '10'}


def test_scsi_eh_deadline_host2():
    context = context_wrap(SCSI_HOSTS_2, SCSI_HOST2_PATH)
    r = SCSIEhDead(context)
    assert r.scsi_host == 'host2'
    assert r.host_eh_deadline == '-1'
    assert r.data == {'host2': '-1'}


def test_scsi_fwver_doc_examples():
    env = {
            'scsi_obj0': SCSIEhDead(context_wrap(SCSI_HOSTS_0, SCSI_HOST0_PATH)),
            'scsi_obj1': SCSIEhDead(context_wrap(SCSI_HOSTS_1, SCSI_HOST1_PATH)),
            'scsi_obj2': SCSIEhDead(context_wrap(SCSI_HOSTS_2, SCSI_HOST2_PATH))
    }
    failed, total = doctest.testmod(scsi_eh_deadline, globs=env)
    assert failed == 0



"""
Grubby
======
Combiner for command ``/usr/sbin/grubby`` parsers.

This combiner uses the parsers:
:class:`insights.parsers.grubby.GrubbyDefaultIndex`,
:class:`insights.parsers.grubby.GrubbyInfoAll`,
:class:`insights.parsers.grubenv.GrubEnv`.
"""

import re

from insights.core.exceptions import ParseException
from insights.core.filters import add_filter
from insights.core.plugins import combiner
from insights.parsers import parse_cmdline_args
from insights.parsers.grub_conf import BootEntry
from insights.parsers.grubby import GrubbyDefaultIndex, GrubbyInfoAll
from insights.parsers.grubenv import GrubEnv
from insights.parsers.ls import LSlanFiltered
from insights.parsers.ls_sys_firmware import LsSysFirmware
from insights.specs import Specs

add_filter(Specs.ls_lan_filtered_dirs, '/sys/firmware')
add_filter(Specs.ls_lan_filtered, ['/sys/firmware', 'efi'])


@combiner([LSlanFiltered, LsSysFirmware])
class IsUEFIBoot(object):
    """
    Tell if the host is boot with UEFI by computing on the files under
    /sys/firmware directory.

    Attributes:
        is_uefi_boot (bool): if the host is boot with UEFI

    Raises:
        ParseException: when failing on the computing of /sys/firmware file
    """

    def __init__(self, ls_lan, ls_sf):
        sf_dir = '/sys/firmware'
        sf_efi_dir = '/sys/firmware/efi'
        ls = ls_lan or ls_sf
        self.is_uefi_boot = (
            sf_efi_dir in ls or (sf_dir in ls and ls.dir_contains(sf_dir, 'efi')) if ls else False
        )

    def __bool__(self):
        return self.is_uefi_boot

    __nonzero__ = __bool__


@combiner(GrubbyInfoAll, GrubbyDefaultIndex, optional=[GrubEnv, IsUEFIBoot])
class Grubby(object):
    """
    Combine command "grubby" parsers into one Combiner.

    Attributes:
        boot_entries (list): All boot entries as ``BootEntry`` instances
        default_index (int): The numeric index of the default boot entry
        default_boot_entry (dict): The boot information for default kernel
        default_kernel (str): The path of the default kernel
        is_efi (bool): True if the host is boot with UEFI

    Raises:
        ParseException: when parsing into error.
    """

    def __init__(self, grubby_info_all, grubby_default_index, grubenv, is_uefi):

        self._boot_entries = {
            i: self._gen_boot_entry(e) for i, e in grubby_info_all.boot_entries.items()
        }
        self.default_index = grubby_default_index.default_index

        if self.default_index not in self._boot_entries:
            raise ParseException(
                "DEFAULT index %s not exist in parsed boot_entries: %s"
                % (self.default_index, list(self._boot_entries.keys()))
            )
        self.default_boot_entry = self._boot_entries[self.default_index]

        self.default_kernel = self.default_boot_entry.get("kernel")
        if not self.default_kernel:
            raise ParseException(
                "DEFAULT kernel-path not exist in default-index: %s" % self.default_index
            )

        self.version = self._version = 2  # GRUB2
        self.is_efi = bool(is_uefi)
        self._kernel_initrds = None  # lazy load
        self._is_kdump_iommu_enabled = None  # lazy load
        self._expand_with_grubenv(grubenv) if grubenv else None
        self.boot_entries = list(self._boot_entries[idx] for idx in sorted(self._boot_entries))

    def _gen_boot_entry(self, entry_data):
        new_entry_data = {
            'name': entry_data.get('title', ''),
            'cmdline': " ".join(
                [
                    entry_data.get('kernel', ''),
                    "root=%s" % entry_data["root"] if entry_data.get('root') else '',
                    entry_data.get('raw_args', ''),
                ]
            ).strip(),  # compatiable for GrubConf usage
        }
        new_entry_data.update(entry_data)
        if entry_data.get('root') and entry_data.get('args'):
            new_entry_data['args'] = parse_cmdline_args(new_entry_data['cmdline'])
            new_entry_data.pop('raw_args')
        return BootEntry(new_entry_data)

    def _expand_with_grubenv(self, grubenv):
        """
        If grubenv, expand the variables for boot_entries:
            - $kernelopts, $tuned_params, $tuned_initrd
        """
        for entry in self._boot_entries.values():
            if_reload_args = False
            if "$kernelopts" in entry.get('args', {}):
                entry['cmdline'] = re.sub(
                    "\\$kernelopts", grubenv.get("kernelopts", ""), entry['cmdline']
                ).strip()
                if_reload_args = True
            if "$tuned_params" in entry.get('args', {}):
                entry['cmdline'] = re.sub(
                    "\\$tuned_params", grubenv.get("tuned_params", ""), entry['cmdline']
                ).strip()
                if_reload_args = True
            if if_reload_args:
                entry['args'] = parse_cmdline_args(entry['cmdline'])

            if "$tuned_initrd" in entry.get('initrd', ''):
                entry['initrd'] = re.sub(
                    "\\$tuned_initrd", grubenv.get("tuned_initrd", ""), entry['initrd']
                ).strip()

    @property
    def kernel_initrds(self):
        """
        Get the `kernel` and `initrd` files referenced in GRUB2 boot entries

        Returns:
            dict: A dict of defined key `grub_kernels` and `grub_initrds`,
                with a list of `kernel` or `initrd` file names as value.

        .. note::
            This property is for the compatiable usage in combiner `GrubConf`.
            To tell the default kernel path or entry, use the provided
            attribute `default_kernel` or `default_boot_entry` directly.
        """
        if self._kernel_initrds is None:
            grub_kernels = []
            grub_initrds = []
            for entry in self._boot_entries.values():
                _kernel = entry.get('kernel', '')
                if _kernel and 'vmlinuz-' in _kernel:
                    entry_kernel = _kernel.split('vmlinuz-', 1)[-1]
                    grub_kernels.append('vmlinuz-' + entry_kernel) if entry_kernel else None

                _initrd = entry.get('initrd', '')
                split_key = 'initramfs-' if 'initramfs-' in _initrd else 'initrd-'
                if _initrd and split_key in _initrd:
                    entry_initrd = _initrd.split(split_key, 1)[-1]
                    grub_initrds.append(split_key + entry_initrd) if entry_initrd else None

            self._kernel_initrds = {"grub_kernels": grub_kernels, "grub_initrds": grub_initrds}

        return self._kernel_initrds

    @property
    def is_kdump_iommu_enabled(self):
        """
        Does any boot entry have 'intel_iommu=on' set in "cmdline"?

        Returns:
            bool: ``True`` when 'intel_iommu=on' is set, otherwise ``False``
        """
        if self._is_kdump_iommu_enabled is None:
            self._is_kdump_iommu_enabled = False
            for entry in self._boot_entries.values():
                if "intel_iommu=on" in entry.get('cmdline', ''):
                    self._is_kdump_iommu_enabled = True
                    break
        return self._is_kdump_iommu_enabled



"""
RearConf - Configuration files of Rear
======================================

Parsers included in this module are:

RearLocalConf - file ``/etc/rear/local.conf``
---------------------------------------------
RearDefaultConf - file ``/usr/share/rear/conf/default.conf``
------------------------------------------------------------
"""

from insights.core import TextFileOutput
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.rear_local_conf)
class RearLocalConf(TextFileOutput):
    """
    Parses content of "/etc/rear/local.conf".

    Typical content of "/etc/rear/local.conf"::

        BACKUP_RESTORE_MOVE_AWAY_FILES=( /boot/grub/grubenv /boot/grub2/grubenv )


    Examples:
        >>> type(local_conf)
        <class 'insights.parsers.rear_conf.RearLocalConf'>
        >>> local_conf.lines[0] == 'BACKUP_RESTORE_MOVE_AWAY_FILES=( /boot/grub/grubenv /boot/grub2/grubenv )'
        True
    """

    def parse_content(self, content):
        content = get_active_lines(content)
        if not content:
            raise SkipComponent
        else:
            super(RearLocalConf, self).parse_content(content)


@parser(Specs.rear_default_conf)
class RearDefaultConf(TextFileOutput):
    """
    Parses content of "/usr/share/rear/conf/default.conf".

    Typical content of "/usr/share/rear/conf/default.conf"::

        COPY_AS_IS_EXCLUDE=( $VAR_DIR/output/ dev/.udev dev/shm dev/shm/ dev/oracleasm dev/mapper dev/watchdog )


    Examples:
        >>> type(default_conf)
        <class 'insights.parsers.rear_conf.RearDefaultConf'>
        >>> default_conf.lines[0] == 'COPY_AS_IS_EXCLUDE=( $VAR_DIR/output/ dev/.udev dev/shm dev/shm/ dev/oracleasm dev/mapper dev/watchdog )'
        True
    """

    def parse_content(self, content):
        content = get_active_lines(content)
        if not content:
            raise SkipComponent
        else:
            super(RearDefaultConf, self).parse_content(content)



"""
CurrentClockSource - file ``/sys/devices/system/clocksource/clocksource0/current_clocksource``
==============================================================================================

This is a relatively simple parser that reads the
``/sys/devices/system/clocksource/clocksource0/current_clocksource`` file.
As well as reporting the contents of the file in its ``data`` property, it
also provides three properties that are true if the clock source is set to
that value:

    * **is_kvm** - the clock source file contains 'kvm-clock'
    * **is_tsc** - the clock source file contains 'tsc'
    * **is_vmi_timer** - the clock source file contains 'vmi-timer'

Examples:

    >>> cs = shared[CurrentClockSource]
    >>> cs.data
    'tsc'
    >>> cs.is_tsc
    True
"""

from .. import Parser, parser
from insights.specs import Specs


@parser(Specs.current_clocksource)
class CurrentClockSource(Parser):
    """
    The CurrentClockSource parser class.

    Attributes:
        data (str): the content of the current_clocksource file.
    """

    def parse_content(self, content):
        self.data = list(content)[0]

    @property
    def is_kvm(self):
        """
        bool: does the clock source contain 'kvm-clock'?
        """
        return 'kvm-clock' in self.data

    @property
    def is_tsc(self):
        """
        bool: does the clock source contain 'tsc'?
        """
        return 'tsc' in self.data

    @property
    def is_vmi_timer(self):
        """
        bool: does the clock source contain 'vmi-timer'?
        """
        return 'vmi-timer' in self.data



import pytest

from mock.mock import patch

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.parsers.docker_list import DockerListContainers
from insights.parsers.podman_list import PodmanListContainers
from insights.specs.datasources.container import containers_with_shell, running_rhel_containers
from insights.tests import context_wrap

PODMAN_LIST_CONTAINERS_2_UP = """
CONTAINER ID                                                       IMAGE         COMMAND                                            CREATED             STATUS                      PORTS                  NAMES               SIZE
03e2861336a76e29155836113ff6560cb70780c32f95062642993b2b3d0fc216   rhel7_httpd   "/usr/sbin/httpd -DFOREGROUND"                     45 seconds ago      Up 37 seconds               0.0.0.0:8080->80/tcp   angry_saha          796 B (virtual 669.2 MB)
05516ea08b565e37e2a4bca3333af40a240c368131b77276da8dec629b7fe102   bd8638c869ea  "/bin/sh -c 'yum install -y vsftpd-2.2.2-6.el6'"   18 hours ago        Up 18 hours ago                                    tender_rosalind     4.751 MB (virtual 200.4 MB)
""".strip()

DOCKER_LIST_CONTAINERS_1_UP = """
CONTAINER ID                                                       IMAGE         COMMAND                                            CREATED             STATUS                      PORTS                  NAMES               SIZE
d3e2861336a76e29155836113ff6560cb70780c32f95062642993b2b3d0fc216   rhel7_httpd   "/usr/sbin/httpd -DFOREGROUND"                     45 seconds ago      Up 37 seconds               0.0.0.0:8080->80/tcp   angry_saha          796 B (virtual 669.2 MB)
d5516ea08b565e37e2a4bca3333af40a240c368131b77276da8dec629b7fe102   bd8638c869ea  "/bin/sh -c 'yum install -y vsftpd-2.2.2-6.el6'"   18 hours ago        Exited (137) 18 hours ago                          tender_rosalind     4.751 MB (virtual 200.4 MB)
""".strip()

FEDORA = """
Fedora release 23 (Twenty Three)
""".strip()

REDHAT_RELEASE7 = """
Red Hat Enterprise Linux release 7.3
""".strip()

DS_CONTAINERS_WITH_SHELL_RET = [
    ('rhel7_httpd', 'podman', '03e2861336a7'),
    ('bd8638c869ea', 'podman', '05516ea08b56'),
    ('rhel7_httpd', 'docker', 'd3e2861336a7'),
]


def fake_shell_out(cmd, split=True, timeout=None, keep_rc=False, env=None, signum=None):
    tmp_cmd = cmd.strip().split()
    if keep_rc:  # fake for ds containers_with_shell
        if '05516ea08b56' in tmp_cmd[2]:  # for test cov
            raise Exception
        if 'podman' in tmp_cmd[0]:
            return [0, "abc"]
        if 'docker' in tmp_cmd[0]:
            return [125, "err"]
    else:  # fake for ds running_rhel_containers
        if '05516ea08b56' in tmp_cmd[4]:  # for test cov
            raise Exception
        if 'podman' in tmp_cmd[0]:
            return [
                REDHAT_RELEASE7,
            ]
        if 'docker' in tmp_cmd[0]:
            return [
                FEDORA,
            ]
    raise Exception()


# ### Test on containers_with_shell


@patch("insights.core.context.HostContext.shell_out", return_value=[0, "abc"])
def test_get_containers_with_shell_both_ok(fso):
    p_ctn = PodmanListContainers(context_wrap(PODMAN_LIST_CONTAINERS_2_UP))
    d_ctn = DockerListContainers(context_wrap(DOCKER_LIST_CONTAINERS_1_UP))
    assert p_ctn is not None
    assert d_ctn is not None

    broker = {PodmanListContainers: p_ctn, DockerListContainers: d_ctn, HostContext: HostContext()}

    ret = containers_with_shell(broker)
    assert len(ret) == 3
    assert ('rhel7_httpd', 'podman', '03e2861336a7') in ret
    assert ('bd8638c869ea', 'podman', '05516ea08b56') in ret
    assert ('rhel7_httpd', 'docker', 'd3e2861336a7') in ret
    # the stopped container is not collected


@patch("insights.core.context.HostContext.shell_out", side_effect=fake_shell_out)
def test_get_containers_with_shell_podman_only(fso):
    p_ctn = PodmanListContainers(context_wrap(PODMAN_LIST_CONTAINERS_2_UP))
    d_ctn = DockerListContainers(context_wrap(DOCKER_LIST_CONTAINERS_1_UP))
    assert p_ctn is not None
    assert d_ctn is not None

    broker = {PodmanListContainers: p_ctn, DockerListContainers: d_ctn, HostContext: HostContext()}

    ret = containers_with_shell(broker)
    assert len(ret) == 1
    assert ('rhel7_httpd', 'podman', '03e2861336a7') in ret
    # container 05516ea08b56 be patched to raise error
    # docker container is from Fedora image, not collected


@patch("insights.core.context.HostContext.shell_out", return_value=[0, "abc"])
def test_get_containers_with_shell_skip_dup(fso):
    p_ctn = PodmanListContainers(context_wrap(PODMAN_LIST_CONTAINERS_2_UP))
    # use the 'podman list' result as input for docker
    d_ctn = DockerListContainers(context_wrap(PODMAN_LIST_CONTAINERS_2_UP))
    assert p_ctn is not None
    assert d_ctn is not None

    broker = {PodmanListContainers: p_ctn, DockerListContainers: d_ctn, HostContext: HostContext()}

    ret = containers_with_shell(broker)
    assert len(ret) == 2
    assert ('rhel7_httpd', 'podman', '03e2861336a7') in ret
    assert ('bd8638c869ea', 'podman', '05516ea08b56') in ret
    # duplicated container is removed from docker, not collected


@patch("insights.core.context.HostContext.shell_out", return_value=[125, "err"])
def test_get_containers_with_shell_none(fso):
    p_ctn = PodmanListContainers(context_wrap(PODMAN_LIST_CONTAINERS_2_UP))
    d_ctn = DockerListContainers(context_wrap(DOCKER_LIST_CONTAINERS_1_UP))
    assert p_ctn is not None
    assert d_ctn is not None

    broker = {PodmanListContainers: p_ctn, DockerListContainers: d_ctn, HostContext: HostContext()}

    with pytest.raises(SkipComponent):
        ret = containers_with_shell(broker)
        assert len(ret) == 0


@patch("insights.core.context.HostContext.shell_out", return_value=[0, "abc"])
def test_get_containers_with_shell_empty(fso):
    broker = {HostContext: HostContext()}

    with pytest.raises(SkipComponent):
        ret = containers_with_shell(broker)
        assert len(ret) == 0


# ### Test on running_rhel_containers


@patch(
    "insights.core.context.HostContext.shell_out",
    return_value=[
        REDHAT_RELEASE7,
    ],
)
def test_get_running_rhel_containers_all_rhel(fso):
    broker = {containers_with_shell: DS_CONTAINERS_WITH_SHELL_RET, HostContext: HostContext()}

    ret = running_rhel_containers(broker)
    assert len(ret) == 3
    assert ('rhel7_httpd', 'podman', '03e2861336a7') in ret
    assert ('bd8638c869ea', 'podman', '05516ea08b56') in ret
    assert ('rhel7_httpd', 'docker', 'd3e2861336a7') in ret


@patch("insights.core.context.HostContext.shell_out", side_effect=fake_shell_out)
def test_get_running_rhel_containers_some_rhel(fso):
    broker = {containers_with_shell: DS_CONTAINERS_WITH_SHELL_RET, HostContext: HostContext()}

    ret = running_rhel_containers(broker)
    assert len(ret) == 1
    assert ('rhel7_httpd', 'podman', '03e2861336a7') in ret
    # container 05516ea08b56 be patched to raise error


@patch(
    "insights.core.context.HostContext.shell_out",
    return_value=[
        FEDORA,
    ],
)
def test_get_running_rhel_containers_none_rhel(fso):
    broker = {containers_with_shell: DS_CONTAINERS_WITH_SHELL_RET, HostContext: HostContext()}

    with pytest.raises(SkipComponent):
        ret = running_rhel_containers(broker)
        assert len(ret) == 0



"""
SystemD service configuration
=============================

Service systemd files are in ``/usr/lib/systemd/system/`` or ``/etc/systemd/``,
and their content format is similar to a '.ini' file.

Parsers included in this module are:

SystemdDocker - file ``/usr/lib/systemd/system/docker.service``
---------------------------------------------------------------

SystemdLogindConf - file ``/etc/systemd/logind.conf``
-----------------------------------------------------

SystemdRpcbindSocketConf - unit file ``rpcbind.socket``
-------------------------------------------------------

SystemdDnsmasqServiceConf - unit file ``dnsmasq.service``
---------------------------------------------------------

SystemdOpenshiftNode - file ``/usr/lib/systemd/system/atomic-openshift-node.service``
-------------------------------------------------------------------------------------

SystemdSystemConf - file ``/etc/systemd/system.conf``
-----------------------------------------------------

SystemdOriginAccounting - file ``/etc/systemd/system.conf.d/origin-accounting.conf``
------------------------------------------------------------------------------------
"""

from insights.core import ConfigParser, LegacyItemAccess
from insights.core.plugins import parser
from insights.parsr import iniparser
from insights.specs import Specs
from insights import CommandParser
from insights.util import deprecated


class SystemdConf(CommandParser, LegacyItemAccess, ConfigParser):
    """
    Base class for parsing systemd INI like configuration files

    """

    def parse_doc(self, content):
        return iniparser.parse_doc("\n".join(content), self)

    def parse_content(self, content):
        super(SystemdConf, self).parse_content(content)
        dict_all = {}
        for section in self.doc:
            section_dict = {}
            option_names = set(o.name for o in section)
            for name in option_names:
                options = [str(o.value) for o in section[name]]
                section_dict[name] = options[0] if len(options) == 1 else options
            dict_all[section.name] = section_dict
        self.data = dict_all


@parser(Specs.systemd_docker)
class SystemdDocker(SystemdConf):
    """
    Class for docker service systemd configuration.

    Typical content of ``/usr/lib/systemd/system/docker.service`` file is::

        [Service]
        Type=notify
        EnvironmentFile=-/etc/sysconfig/docker
        EnvironmentFile=-/etc/sysconfig/docker-storage
        EnvironmentFile=-/etc/sysconfig/docker-network
        Environment=GOTRACEBACK=crash
        ExecStart=/bin/sh -c '/usr/bin/docker-current daemon \\
            --authorization-plugin=rhel-push-plugin \\
            --exec-opt native.cgroupdriver=systemd \\
            $OPTIONS \\
            $DOCKER_STORAGE_OPTIONS \\
            $DOCKER_NETWORK_OPTIONS \\
            $ADD_REGISTRY \\
            $BLOCK_REGISTRY \\
            $INSECURE_REGISTRY \\
            2>&1 | /usr/bin/forward-journald -tag docker'
        LimitNOFILE=1048576

    Example:
        >>> docker_service["Service"]["ExecStart"].endswith("-tag docker'")
        True
        >>> len(docker_service["Service"]["EnvironmentFile"])
        3
    """

    def __init__(self, *args, **kwargs):
        deprecated(SystemdDocker, "This Parser is deprecated", "3.7.0")
        super(SystemdDocker, self).__init__(*args, **kwargs)

    pass


@parser(Specs.systemd_system_conf)
class SystemdSystemConf(SystemdConf):
    """
    Class for systemd master configuration in the ``/etc/systemd/system.conf``
    file.

    Typical content of the ``/etc/systemd/system.conf`` file is::

        [Manager]
        RuntimeWatchdogSec=0
        ShutdownWatchdogSec=10min

    Example:
        >>> system_conf["Manager"]["RuntimeWatchdogSec"]
        '0'
    """

    pass


@parser(Specs.systemd_system_origin_accounting)
class SystemdOriginAccounting(SystemdConf):
    """
    Class for systemd master configuration in the ``/etc/systemd/system.conf.d/origin-accounting.conf``
    file.

    Typical content of the ``/etc/systemd/system.conf.d/origin-accounting.conf`` file is::

        [Manager]
        DefaultCPUAccounting=yes
        DefaultMemoryAccounting=yes
        DefaultBlockIOAccounting=yes

    Example:
        >>> system_origin_accounting["Manager"]["DefaultCPUAccounting"]
        'True'
    """

    pass


@parser(Specs.systemd_openshift_node)
class SystemdOpenshiftNode(SystemdConf):
    """
    Class for atomic-openshift-node systemd configuration.

    Typical output of
    ``/usr/lib/systemd/system/atomic-openshift-node.service`` file is::

        [Service]
        Type=notify
        RestartSec=5s
        OOMScoreAdjust=-999
        ExecStartPost=/usr/bin/sleep 10
        ExecStartPost=/usr/sbin/sysctl --system

    Example:
        >>> openshift_node_service["Service"]["RestartSec"]
        '5s'
        >>> len(openshift_node_service["Service"]["ExecStartPost"])
        2
    """

    pass


@parser(Specs.systemd_logind_conf)
class SystemdLogindConf(SystemdConf):
    """
    Class for systemd logind configuration.

    Typical content of the ``/etc/systemd/logind.conf`` file is::

        [Login]
        ReserveVT=6
        KillUserProcesses=Yes
        RemoveIPC=no

    Example:
        >>> logind_conf["Login"]["ReserveVT"]
        '6'
        >>> logind_conf["Login"]["KillUserProcesses"]  # 'Yes' turns to 'True'
        'True'
        >>> logind_conf.get("Login").get("RemoveIPC")  # 'no' turns to 'False'
        'False'
    """

    pass


@parser(Specs.systemctl_cat_rpcbind_socket)
class SystemdRpcbindSocketConf(SystemdConf):
    """
    Class for systemd configuration for rpcbind.socket unit.

    Typical content of the ``rpcbind.socket`` unit file is::

        [Unit]
        Description=RPCbind Server Activation Socket
        DefaultDependencies=no
        Wants=rpcbind.target
        Before=rpcbind.target

        [Socket]
        ListenStream=/run/rpcbind.sock

        # RPC netconfig can't handle ipv6/ipv4 dual sockets
        BindIPv6Only=ipv6-only
        ListenStream=0.0.0.0:111
        ListenDatagram=0.0.0.0:111
        ListenStream=[::]:111
        ListenDatagram=[::]:111

        [Install]
        WantedBy=sockets.target

    Example:
        >>> rpcbind_socket["Socket"]["ListenStream"]
        ['/run/rpcbind.sock', '0.0.0.0:111', '[::]:111']
    """

    pass


@parser(Specs.systemctl_cat_dnsmasq_service)
class SystemdDnsmasqServiceConf(SystemdConf):
    """
    Class for systemd configuration for dnsmasq.service unit.

    Typical content of the ``dnsmasq.service`` unit file is::

        [Unit]
        Description=DNS caching server.
        After=network.target

        [Service]
        ExecStart=/usr/sbin/dnsmasq -k

        [Install]
        WantedBy=multi-user.target

    Example:
        >>> dnsmasq_service["Unit"]["After"]
        'network.target'
    """

    pass



from __future__ import print_function
import six
import sys
import inspect
from collections import namedtuple

from pprint import pprint
from six import StringIO
from insights import _COLOR, dr, datasource, rule, condition, incident, parser
from insights.core.context import ExecutionContext
from insights.formats import Formatter, FormatterAdapter, render


try:
    from colorama import init

    if _COLOR == "always":
        from colorama import Fore, Style

        init(strip=False)
    elif _COLOR == "auto":
        from colorama import Fore, Style

        init()
    elif _COLOR == "never":

        class Default(type):
            def __getattr__(*args):
                return ""

        class Fore(six.with_metaclass(Default)):
            pass

        class Style(six.with_metaclass(Default)):
            pass

except ImportError:
    print("Install colorama if console colors are preferred.")

    class Default(type):
        def __getattr__(*args):
            return ""

    class Fore(six.with_metaclass(Default)):
        pass

    class Style(six.with_metaclass(Default)):
        pass


def _find_context(broker):
    for k, v in broker.instances.items():
        if inspect.isclass(k) and issubclass(k, ExecutionContext):
            return v


def render_links(component):
    links = dr.get_delegate(component).links or {}
    if any(links.values()):
        space = " " * 4
        dbl_space = space * 2
        output = StringIO()
        output.write("Links:\n")
        for key in sorted(links):
            values = sorted(links[key])
            if values:
                output.write(space + key + ":\n")
                for v in values:
                    output.write(dbl_space + v + "\n")
        output.seek(0)
        return output.read()
    return ""


class HumanReadableFormat(Formatter):
    """
    This class prints a human readable summary of rule hits. It should be used
    as a context manager and given an instance of an
    ``insights.core.dr.Broker``. ``dr.run`` should be called within the context
    using the same broker.

    Args:
        broker (Broker): the broker to watch and provide a summary about.
        missing (bool): shows rules with missing dependencies if True.
        tracebacks (bool): shows tracebacks if any exceptions were raised.
            Useful for debugging.
        dropped (bool): Shows any files that weren't collected if running
            against an archive. Included for a corner case and typically not
            used in general.
        stream (file-like): Output is written to stream. Defaults to sys.stdout.
    """

    def __init__(
        self,
        broker,
        missing=False,
        tracebacks=False,
        dropped=False,
        show_rules=None,
        no_details=False,
        stream=sys.stdout,
    ):
        super(HumanReadableFormat, self).__init__(broker, stream=stream)
        self.missing = missing
        self.tracebacks = tracebacks
        self.dropped = dropped
        self.show_rules = [] if show_rules is None else show_rules
        self.no_details = no_details

    def print_header(self, header, color):
        ln = len(header)
        print(color + '-' * ln, file=self.stream)
        print(header, file=self.stream)
        print('-' * ln + Style.RESET_ALL, file=self.stream)

    def preprocess(self):
        response = namedtuple('response', 'color label intl title')
        self.responses = {
            'skip': response(color=Fore.BLUE, label="SKIP", intl='S', title="Missing Deps: "),
            'pass': response(color=Fore.GREEN, label="PASS", intl='P', title="Passed      : "),
            'rule': response(color=Fore.RED, label="FAIL", intl='F', title="Failed      : "),
            'info': response(color=Fore.WHITE, label="INFO", intl='I', title="Info        : "),
            'none': response(
                color=Fore.BLUE, label="RETURNED NONE", intl='N', title="Ret'd None  : "
            ),
            'metadata': response(color=Fore.YELLOW, label="META", intl='M', title="Metadata    : "),
            'metadata_key': response(
                color=Fore.MAGENTA, label="META", intl='K', title="Metadata Key: "
            ),
            'fingerprint': response(
                color=Fore.YELLOW, label="FINGERPRINT", intl='P', title="Fingerprint : "
            ),
            'exception': response(color=Fore.RED, label="EXCEPT", intl='E', title="Exceptions  : "),
        }

        self.counts = {}
        for key in self.responses:
            self.counts[key] = 0

        self.print_header("Progress:", Fore.CYAN)
        self.broker.add_observer(self.progress_bar, rule)
        self.broker.add_observer(self.progress_bar, condition)
        self.broker.add_observer(self.progress_bar, incident)
        self.broker.add_observer(self.progress_bar, parser)

    def progress_bar(self, c, broker):
        """
        Print the formated progress information for the processed return types
        """
        v = broker.get(c)

        if v and isinstance(v, dict) and len(v) > 0 and 'type' in v:
            if v["type"] in self.responses:
                print(
                    self.responses[v["type"]].color
                    + self.responses[v["type"]].intl
                    + Style.RESET_ALL,
                    end="",
                    file=self.stream,
                )
            else:
                print(".", end="", file=self.stream)
        elif c in broker.exceptions:
            self.counts['exception'] += len(broker.exceptions[c])
            print(Fore.RED + "E" + Style.RESET_ALL, end="", file=self.stream)
        return self

    def show_tracebacks(self):
        """Show tracebacks"""
        if self.broker.tracebacks:
            print(file=self.stream)
            print("Tracebacks:", file=self.stream)
            for t in self.broker.tracebacks.values():
                print(t, file=self.stream)

    def show_dropped(self):
        """Show dropped files"""
        ctx = _find_context(self.broker)
        if ctx and ctx.all_files:
            ds = self.broker.get_by_type(datasource)
            vals = []
            for v in ds.values():
                if isinstance(v, list):
                    vals.extend(d.path for d in v)
                else:
                    vals.append(v.path)
            dropped = set(ctx.all_files) - set(vals)
            pprint("Dropped Files:", stream=self.stream)
            pprint(dropped, indent=4, stream=self.stream)

    def show_description(self):
        """Prints the formatted response for the matching return type"""

        def printit(c, v):
            resp = self.responses[v["type"]]
            name = "[%s] %s" % (resp.label, dr.get_name(c))
            underline = "-" * len(name)
            name = "%s%s%s" % (resp.color, name, Style.RESET_ALL)
            print(name, file=self.stream)
            if self.no_details:
                return
            print(underline, file=self.stream)
            if v.get('type') != 'none':
                print(render_links(c), file=self.stream)
                print(render(c, v), file=self.stream)
            print(file=self.stream)

        for c in sorted(self.broker.get_by_type(rule), key=dr.get_name):
            v = self.broker[c]
            _type = v.get('type')
            if _type is None:
                continue

            if _type in self.responses:
                self.counts[_type] += 1

            if (
                (self.missing and _type == 'skip')
                or (self.show_rules and _type in self.show_rules)
                or (not self.show_rules and _type not in ['skip', 'none'])
            ):
                printit(c, v)

        print(file=self.stream)

        self.print_header("Rule Execution Summary", Fore.CYAN)
        for c in self.counts:
            print(
                self.responses[c].color
                + self.responses[c].title
                + str(self.counts[c])
                + Style.RESET_ALL,
                file=self.stream,
            )

    def postprocess(self):
        if self.tracebacks:
            self.show_tracebacks()
        if self.dropped:
            self.show_dropped()

        print(file=self.stream)
        print(file=self.stream)
        self.print_header("Rules Executed", Fore.CYAN)
        self.show_description()


class HumanReadableFormatAdapter(FormatterAdapter):
    """Displays results in a human readable format."""

    @staticmethod
    def configure(p):
        p.add_argument("-t", "--tracebacks", help="Show stack traces.", action="store_true")
        p.add_argument(
            "-d",
            "--dropped",
            help="Show collected files that weren't processed.",
            action="store_true",
        )
        p.add_argument("-m", "--missing", help="Show missing requirements.", action="store_true")
        p.add_argument(
            "-S",
            "--show-rules",
            default=[],
            nargs="+",
            choices=["fail", "info", "pass", "none", "metadata", "fingerprint"],
            metavar="TYPE",
            help="Show results per rule's type: 'fail', 'info', 'pass', 'none', 'metadata', and 'fingerprint'. This option must not appear immediately before the archive path argument.",
        )
        p.add_argument(
            "-F",
            "--fail-only",
            help="Show FAIL results only. Conflict with '-m', will be dropped when using them together. This option is deprecated by '-S fail'",
            action="store_true",
        )
        p.add_argument(
            "--no-details",
            help="Hide the details and show result only, for TEXT format only",
            action="store_true",
        )

    def __init__(self, args=None):
        self.tracebacks = args.tracebacks
        self.dropped = args.dropped
        self.missing = args.missing
        self.no_details = args.no_details
        fail_only = args.fail_only
        if self.missing and fail_only:
            print(Fore.YELLOW + 'Options conflict: -m and -F, drops -F', file=sys.stderr)
            fail_only = None
        self.show_rules = []  # Empty by default, means show ALL types (excludes "none")
        if not args.show_rules and fail_only:
            self.show_rules = ['rule']
        elif args.show_rules:
            self.show_rules = [opt.replace('fail', 'rule') for opt in args.show_rules]

    def preprocess(self, broker):
        self.formatter = HumanReadableFormat(
            broker,
            self.missing,
            self.tracebacks,
            self.dropped,
            self.show_rules,
            self.no_details,
        )
        self.formatter.preprocess()

    def postprocess(self, broker):
        self.formatter.postprocess()



import doctest
import pytest

from insights.combiners import redhat_release as rr
from insights.combiners.redhat_release import RedHatRelease, serialize_RedHatRelease, deserialize_RedHatRelease
from insights.core.exceptions import SkipComponent
from insights.parsers.redhat_release import RedhatRelease as RRParser
from insights.parsers.uname import Uname
from insights.tests import context_wrap

UNAME = "Linux localhost.localdomain 3.10.0-327.rt56.204.el7.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
BAD_UNAME = "Linux localhost.localdomain 2.6.24.7-101.el5rt.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_10 = "Linux localhost.localdomain 6.12.0-55.14.1.el10_0.x86_64 #1 SMP PREEMPT_DYNAMIC Mon May 26 04:24:13 EDT 2025 x86_64 GNU/Linux"

REDHAT_RELEASE = """
Red Hat Enterprise Linux Server release 7.2 (Maipo)
""".strip()

FEDORA = """
Fedora release 23 (Twenty Three)
""".strip()

REDHAT_RELEASE_8_CONTAINER_1 = """
Red Hat Enterprise Linux Server release 8.4 (Ootpa)
""".strip()

REDHAT_RELEASE_8_CONTAINER_2 = """
Red Hat Enterprise Linux Server release 8.6 (Ootpa)
""".strip()

REDHAT_RELEASE_10_0 = """
Red Hat Enterprise Linux release 10.0 (Coughlan)
""".strip()


def test_RedHatRelease_uname():
    un = Uname(context_wrap(UNAME))
    expected = (7, 2)
    result = RedHatRelease(un, None)
    assert result.major == expected[0]
    assert result.minor == expected[1]
    assert result.rhel == result.rhel7 == '7.2'
    assert result.rhel6 is None


def test_RedHatRelease_uname_10():
    un = Uname(context_wrap(UNAME_10))
    expected = (10, 0)
    result = RedHatRelease(un, None)
    assert result.major == expected[0]
    assert result.minor == expected[1]
    assert result.rhel == result.rhel10 == '10.0'
    assert result.rhel9 is None


def test_RedHatRelease_redhat_release():
    rel = RRParser(context_wrap(REDHAT_RELEASE))
    expected = (7, 2)
    result = RedHatRelease(None, rel)
    assert result.major == expected[0]
    assert result.minor == expected[1]
    assert result.rhel == result.rhel7 == '7.2'
    assert result.rhel8 is None
    assert result.rhel9 is None


def test_RedHatRelease_both():
    un = Uname(context_wrap(UNAME))
    rel = RRParser(context_wrap(REDHAT_RELEASE))
    expected = (7, 2)
    result = RedHatRelease(un, rel)
    assert result.major == expected[0]
    assert result.minor == expected[1]
    assert result.rhel == result.rhel7 == '7.2'
    assert result.rhel6 is None
    assert result.rhel8 is None
    assert result.rhel9 is None


def test_RedHatRelease_serialize_deserialize():
    rel = RedHatRelease(None, RRParser(context_wrap(REDHAT_RELEASE_10_0)))
    serialized = serialize_RedHatRelease(rel)
    assert serialized == {
            "major": 10,
            "minor": 0,
            "rhel": "10.0",
            "rhel6": None,
            "rhel7": None,
            "rhel8": None,
            "rhel9": None,
            "rhel10": "10.0",
    }

    deserialized = deserialize_RedHatRelease(RedHatRelease, serialized)
    expected = (10, 0)
    assert deserialized.major == expected[0]
    assert deserialized.minor == expected[1]
    assert deserialized.rhel6 is None
    assert deserialized.rhel7 is None
    assert deserialized.rhel8 is None
    assert deserialized.rhel9 is None
    assert deserialized.rhel == deserialized.rhel10 == '10.0'


def test_raise():
    un = Uname(context_wrap(BAD_UNAME))

    with pytest.raises(SkipComponent):
        RedHatRelease(un, None)

    with pytest.raises(SkipComponent):
        RedHatRelease(None, None)


def test_doc_examples():
    env = {
            'rh_rel': RedHatRelease(Uname(context_wrap(UNAME)), None),
          }
    failed, total = doctest.testmod(rr, globs=env)
    assert failed == 0



# -*- coding: UTF-8 -*-

from insights.client.phase.v1 import update
from mock.mock import patch
from pytest import raises


@patch("insights.client.phase.v1.InsightsClient")
@patch("insights.client.phase.v1.InsightsConfig")
def test_update_payload_on(insights_config, insights_client):
    """
    Rules are not updated when a payload is uploaded
    """
    insights_config.return_value.load_all.return_value.payload = True
    try:
        update()
    except SystemExit:
        pass
    insights_client.return_value.update.assert_called_once()


@patch("insights.client.phase.v1.InsightsClient")
@patch("insights.client.phase.v1.InsightsConfig")
def test_update_payload_off(insights_config, insights_client):
    """
    Rules are updated in normal operation (no payload)
    """
    insights_config.return_value.load_all.return_value.payload = False
    try:
        update()
    except SystemExit:
        pass
    insights_client.return_value.update.assert_called_once()


@patch("insights.client.phase.v1.InsightsClient")
@patch("insights.client.phase.v1.InsightsConfig")
def test_exit_ok(insights_config, insights_client):
    """
    Support collection replaces the normal client run.
    """
    with raises(SystemExit) as exc_info:
        update()
    assert exc_info.value.code == 0


@patch("insights.client.phase.v1.InsightsClient")
@patch("insights.client.phase.v1.InsightsConfig")
@patch("insights.client.phase.v1.logger")
def test_exit_error(logger, insights_config, insights_client):
    from requests.exceptions import ConnectionError

    insights_client.return_value.update.side_effect = ConnectionError("Connection error")
    with raises(SystemExit) as exc_info:
        update()

    assert exc_info.value.code == 101
    logger.error.assert_called_with("Failed to update rules. Error: %s", 'Connection error')



"""
Custom datasources relate to `/sys/fs/cgroup/memory`
"""

import os

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.specs import Specs


class LocalSpecs(Specs):
    """Local specs used only by sys_fs_cgroup_memory_tasks_number datasources"""

    sys_fs_cgroup_memory_tasks_raw = simple_command(
        "/usr/bin/find /sys/fs/cgroup/memory -name 'tasks'"
    )
    """ Returns the output of command ``/usr/bin/find /sys/fs/cgroup/memory -name 'tasks'`` """


@datasource(LocalSpecs.sys_fs_cgroup_memory_tasks_raw, HostContext)
def tasks_number(broker):
    """
    This datasource provides the numeber of "tasks" file collected
    from ``/usr/bin/find /sys/fs/cgroup/memory -name 'tasks'``.

    Typical content of ``/usr/bin/find /sys/fs/cgroup/memory -name 'tasks'`` command is::

        /sys/fs/cgroup/memory/user.slice/tasks
        /sys/fs/cgroup/memory/system.slice/rh-nginx120-nginx.service/tasks
        /sys/fs/cgroup/memory/system.slice/named.service/tasks
        /sys/fs/cgroup/memory/system.slice/rhel-push-plugin.service/tasks

    Returns:
        string: the number of "tasks" file under /sys/fs/cgroup/memory

    Raises:
        SkipComponent: When any exception occurs.
    """
    exceptions = [
        'no such file or directory',
    ]
    content = broker[LocalSpecs.sys_fs_cgroup_memory_tasks_raw].content
    if len(content) == 0 or not any(ex in content[0].lower() for ex in exceptions):
        # cleaner is not required per the content
        return DatasourceProvider(
            content=str(len(content)),
            relative_path='insights_datasources/sys_fs_cgroup_memory_tasks_number',
        )
    raise SkipComponent


@datasource(HostContext)
def uniq_memory_swappiness(broker):
    """
    This datasource reads all the memory.swappiness files under /sys/fs/cgroup/memory directory
    one by one and returns the uniq memory swappiness setting of the all control groups.

    The output of this datasource looks like:
        10   1
        60   66

    Returns:
        str: Returns a multiline string in the format as ``value   count``.

    Raises:
        SkipComponent: When any exception occurs.
    """
    file_name = "memory.swappiness"
    data = {}

    for root, _, files in os.walk("/sys/fs/cgroup/memory"):
        if file_name in files:
            file_path = os.path.join(root, file_name)
            with open(file_path, 'r') as setting:
                key = setting.read().strip()
                data[key] = data.get(key, 0) + 1

    if not data:
        raise SkipComponent()

    data_list = []
    for value, count in data.items():
        data_list.append("{0}   {1}".format(value, count))

    # cleaner is not required per the content
    return DatasourceProvider(
        content="\n".join(data_list),
        relative_path='insights_datasources/sys_fs_cgroup_uniq_memory_swappiness',
    )



import doctest

from insights.parsers.modinfo import KernelModulesInfo
from insights.combiners.modinfo import ModulesInfo
from insights.tests.parsers.test_modinfo import (
        MODINFO_I40E, MODINFO_INTEL, MODINFO_BNX2X
)
from insights.tests import context_wrap
from insights.combiners import modinfo


def test_modinfo_doc_examples():
    filter_modules = KernelModulesInfo(context_wrap(
        '{0}\n{1}\n{2}'.format(
            MODINFO_I40E,
            MODINFO_INTEL,
            MODINFO_BNX2X)
    ))
    combiner_obj = ModulesInfo(filter_modules)
    env = {'modules_obj': combiner_obj}
    failed, total = doctest.testmod(modinfo, globs=env)
    assert failed == 0


def test_modules_info():
    filter_modules = KernelModulesInfo(context_wrap(
        '{0}\n{1}\n{2}'.format(
            MODINFO_I40E,
            MODINFO_INTEL,
            MODINFO_BNX2X)
    ))
    combiner_obj = ModulesInfo(
        filter_modules
    )
    assert 'i40e' in combiner_obj
    assert combiner_obj['i40e']['signer'] == 'Red Hat Enterprise Linux kernel signing key'
    assert len(combiner_obj) == 3
    assert 'abc' not in combiner_obj
    assert 'i40e' in combiner_obj.retpoline_y
